@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: decmain.cpp,v 1.18 2005/09/27 08:44:33 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: decmain.cpp,v 1.25 2007/12/11 02:27:22 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -49,9 +49,6 @@ const char *chroma2string (dirac_chroma_t chroma)
 {
     switch (chroma)
     {
-    case Yonly:
-        return "Y_ONLY";
-
     case format422:
         return "4:2:2";
 
@@ -61,9 +58,6 @@ const char *chroma2string (dirac_chroma_t chroma)
     case format420:
         return "4:2:0";
 
-    case format411:
-        return "4:1:1";
-
     default:
         break;
     }
@@ -71,19 +65,35 @@ const char *chroma2string (dirac_chroma_t chroma)
     return "Unknown";
 }
 
-const char *ftype2string (dirac_frame_type_t ftype)
+const char *ftype2string (dirac_frame_type_t ftype, dirac_reference_type_t rtype)
 {
     switch (ftype)
     {
-    case I_frame:
-        return "I_frame";
+    case INTRA_FRAME:
+    {
+        switch (rtype)
+        {
+            case REFERENCE_FRAME:
+                return "Intra Ref Frame";
+            case NON_REFERENCE_FRAME:
+                return "Intra Non-Ref Frame";
+            default:
+                return "Intra Unknown-Ref Frame";
+        }
+    }
+    case INTER_FRAME:
+    {
+        switch (rtype)
+        {
+            case REFERENCE_FRAME:
+                return "Inter Ref Frame";
+            case NON_REFERENCE_FRAME:
+                return "Inter Non-Ref Frame";
+            default:
+                return "Inter Unknown-Ref Frame";
+        }
+    }
     
-    case L1_frame:
-        return "L1_frame";
-    
-    case L2_frame:
-        return "L2_frame";
-
     default:
         break;
     }
@@ -98,16 +108,13 @@ static void WritePicData (dirac_decoder_t *decoder, FILE *fp)
     assert(decoder->fbuf);
 
     assert(decoder->fbuf->buf[0]);
-    fwrite (decoder->fbuf->buf[0], decoder->seq_params.width*decoder->seq_params.height, 1, fp);
+    fwrite (decoder->fbuf->buf[0], decoder->src_params.width*decoder->src_params.height, 1, fp);
 
-    if (decoder->seq_params.chroma != Yonly)
-    {
-        assert(decoder->fbuf->buf[1]);
-        fwrite (decoder->fbuf->buf[1], decoder->seq_params.chroma_width*decoder->seq_params.chroma_height, 1, fp);
+    assert(decoder->fbuf->buf[1]);
+    fwrite (decoder->fbuf->buf[1], decoder->src_params.chroma_width*decoder->src_params.chroma_height, 1, fp);
 
-        assert(decoder->fbuf->buf[2]);
-        fwrite (decoder->fbuf->buf[2], decoder->seq_params.chroma_width*decoder->seq_params.chroma_height, 1, fp);
-    }
+    assert(decoder->fbuf->buf[2]);
+    fwrite (decoder->fbuf->buf[2], decoder->src_params.chroma_width*decoder->src_params.chroma_height, 1, fp);
 }
 
 
@@ -132,8 +139,8 @@ static void DecodeDirac (const char *iname, const char *oname)
     dirac_decoder_t *decoder = NULL;
     FILE *ifp;
     FILE *fpdata;
-    unsigned char buffer[4096];
-    int bytes;
+    unsigned char buffer[8192];
+    int bytes = 0;
     int num_frames = 0;
     char infile_name[FILENAME_MAX];
     char outfile_hdr[FILENAME_MAX];
@@ -141,10 +148,8 @@ static void DecodeDirac (const char *iname, const char *oname)
     dirac_decoder_state_t state = STATE_BUFFER;
 
     strncpy(infile_name, iname, sizeof(infile_name));
-    strcat(infile_name, ".drc");
 
     strncpy(outfile_data, oname, sizeof(outfile_data));
-    strcat(outfile_data, ".yuv");
 
     if ((ifp = fopen (infile_name, "rb")) ==NULL)
     {
@@ -193,28 +198,30 @@ static void DecodeDirac (const char *iname, const char *oname)
 
             if (verbose)
             {
-                fprintf (stderr, "SEQUENCE : width=%d height=%d chroma=%s chroma_width=%d chroma_height=%d frame_rate=%f, interlace=%s topfieldfirst=%s\n", 
-                decoder->seq_params.width,
-                decoder->seq_params.height,
-                chroma2string(decoder->seq_params.chroma),
-                decoder->seq_params.chroma_width,
-                decoder->seq_params.chroma_height,
-                (float)decoder->seq_params.frame_rate.numerator/
-                decoder->seq_params.frame_rate.denominator,
-                decoder->seq_params.interlace ? "yes" : "no",
-                decoder->seq_params.topfieldfirst ? "yes" : "no");
+                fprintf (stdout, "\nSEQUENCE : major_ver=%d minor_version=%d profile=%d level=%d width=%d height=%d chroma=%s chroma_width=%d chroma_height=%d frame_rate=%d/%d, source_sampling=%s topfieldfirst=%s", 
+                decoder->parse_params.major_ver,
+                decoder->parse_params.minor_ver,
+                decoder->parse_params.profile,
+                decoder->parse_params.level,
+                decoder->src_params.width,
+                decoder->src_params.height,
+                chroma2string(decoder->src_params.chroma),
+                decoder->src_params.chroma_width,
+                decoder->src_params.chroma_height,
+                decoder->src_params.frame_rate.numerator,
+                decoder->src_params.frame_rate.denominator,
+                decoder->src_params.source_sampling == 0 ? "progressive" : 
+                ( decoder->src_params.source_sampling == 1 ? "interlaced" : "UNKNOWN" ),
+                decoder->src_params.topfieldfirst ? "yes" : "no");
             }
 
             FreeFrameBuffer(decoder);
 
             buf[0] = buf[1] = buf[2] = 0;
 
-            buf[0] = (unsigned char *)malloc (decoder->seq_params.width * decoder->seq_params.height);
-            if (decoder->seq_params.chroma != Yonly)
-            {
-                buf[1] = (unsigned char *)malloc (decoder->seq_params.chroma_width * decoder->seq_params.chroma_height);
-                buf[2] = (unsigned char *)malloc (decoder->seq_params.chroma_width * decoder->seq_params.chroma_height);
-            }
+            buf[0] = (unsigned char *)malloc (decoder->src_params.width * decoder->src_params.height);
+            buf[1] = (unsigned char *)malloc (decoder->src_params.chroma_width * decoder->src_params.chroma_height);
+            buf[2] = (unsigned char *)malloc (decoder->src_params.chroma_width * decoder->src_params.chroma_height);
             dirac_set_buf (decoder, buf, NULL);
 
          
@@ -226,7 +233,7 @@ static void DecodeDirac (const char *iname, const char *oname)
             * End of Sequence detected. Free the frame buffers
             */
             if (verbose)
-                fprintf (stderr, "SEQUENCE_END\n");
+                fprintf (stdout, "\nSEQUENCE_END");
             
             FreeFrameBuffer(decoder);
             break;
@@ -236,18 +243,17 @@ static void DecodeDirac (const char *iname, const char *oname)
             * Start of frame detected. If decoder is too slow and frame can be
             * skipped, inform the parser to skip decoding the frame
             */
-            num_frames++;
             if (verbose)
             {
-                fprintf (stderr, "PICTURE_START : frame_type=%s frame_num=%d\n",
-                    ftype2string(decoder->frame_params.ftype),
+                fprintf (stdout, "\nPICTURE_START : frame_type=%s frame_num=%d",
+                    ftype2string(decoder->frame_params.ftype, decoder->frame_params.rtype),
                     decoder->frame_params.fnum);
             }
-            /* Just for testing skip every L2_frame */
-            if (skip && decoder->frame_params.ftype == L2_frame)
+            /* Just for testing skip every Non-reference frame */
+            if (skip && decoder->frame_params.rtype == NON_REFERENCE_FRAME)
             {
                 if (verbose)
-                    fprintf (stderr, "              : Skipping frame\n");
+                    fprintf (stdout, "\n              : Skipping frame");
 
                 dirac_skip (decoder, 1);
             }
@@ -256,10 +262,11 @@ static void DecodeDirac (const char *iname, const char *oname)
             break;
 
         case STATE_PICTURE_AVAIL:
+            num_frames++;
             if (verbose)
             {
-                fprintf (stderr, "PICTURE_AVAIL : frame_type=%s frame_num=%d\n",
-                    ftype2string(decoder->frame_params.ftype),
+                fprintf (stdout, "\nPICTURE_AVAIL : frame_type=%s frame_num=%d",
+                    ftype2string(decoder->frame_params.ftype, decoder->frame_params.rtype), 
                     decoder->frame_params.fnum);
             }
             /* picture available for display */
@@ -278,7 +285,7 @@ static void DecodeDirac (const char *iname, const char *oname)
     stop_t=clock();
 
     if ( verbose )
-        fprintf (stdout, "Time per frame: %g\n",
+        fprintf (stdout, "\nTime per frame: %g",
                 (double)(stop_t-start_t)/(double)(CLOCKS_PER_SEC*num_frames));
 
     fclose(fpdata);
