@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: band_codec.h,v 1.15 2005/12/02 10:16:52 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: band_codec.h,v 1.26 2007/07/26 12:46:35 tjdwave Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -23,6 +23,7 @@
 * Contributor(s): Thomas Davies (Original Author),
 *                 Scott R Ladd,
 *                 Steve Bearcroft
+*                 Andrew Kennedy
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -43,9 +44,12 @@
 #include <libdirac_common/arith_codec.h>
 #include <libdirac_common/wavelet_utils.h>
 
+
 namespace dirac
 {
 
+   class SubbandByteIO;
+
     //Subclasses the arithmetic codec to produce a coding/decoding tool for subbands
 
 
@@ -53,87 +57,62 @@ namespace dirac
     /*!
         A general class for coding and decoding wavelet subband data, deriving from the abstract ArithCodec class.
      */
-    class BandCodec: public ArithCodec<PicArray >
+    class BandCodec: public ArithCodec<CoeffArray>
     {
     public:
 
-        //! Constructor for encoding.
+        //! Constructor 
         /*!
             Creates a BandCodec object to encode subband data
-            \param    bits_out    the output for the encoded bits
+            \param    subband_byteio   input/output for the encoded bits
             \param    number_of_contexts the number of contexts used in the encoding process
             \param    band_list    the set of all the subbands
-            \param     band_num    the number of the subband being coded 
+            \param    band_num    the number of the subband being coded 
+            \param    is_intra    Flag indicating whether the band comes from an intra frame
          */
-        BandCodec(BasicOutputManager* bits_out,
+        BandCodec(SubbandByteIO* subband_byteio,
                   size_t number_of_contexts,
                   const SubbandList& band_list,
-                  int band_num);
-
-        //! Constructor for decoding.
-        /*!
-            Creates a BandCodec object to decode subband data.
-            \param    bits_in        the input for the encoded bits
-            \param    number_of_contexts the number of contexts used in the decoding process
-            \param    band_list    the set of all the subbands
-            \param     band_num    the number of the subband being decoded 
-         */
-        BandCodec(BitInputManager* bits_in,
-                  size_t number_of_contexts,
-                  const SubbandList& band_list,
-                  int band_num);
-
-        //! Initialise the contexts according to predefined counts.
-        void InitContexts();
+                  int band_num,
+                  const bool is_intra);
 
     protected:
         //! Code an individual quantised value and perform inverse-quantisation
-        void CodeVal( PicArray& in_data , const int xpos , const int ypos , const ValueType val);
+        inline void CodeVal( CoeffArray& in_data , const int xpos , const int ypos , const CoeffType val);
 
         //! Decode an individual quantised value and perform inverse-quantisation
-        void DecodeVal(PicArray& out_data , const int xpos , const int ypos );
+        inline void DecodeVal(CoeffArray& out_data , const int xpos , const int ypos );
+
+        //! Encode the offset for a code block quantiser
+        void CodeQIndexOffset( const int offset );
+
+        //! Decode the offset for a code block quantiser
+        int DecodeQIndexOffset();
 
         //! Set a code block area to a given value
-        inline void SetToVal( const CodeBlock& code_block , PicArray& pic_data , const ValueType val);
+        inline void SetToVal( const CodeBlock& code_block , CoeffArray& coeff_data , const CoeffType val);
 
         //! Set all block values to 0
-        inline void ClearBlock( const CodeBlock& code_block , PicArray& pic_data);
+        inline void ClearBlock( const CodeBlock& code_block , CoeffArray& coeff_data);
 
     private:
         //functions
         // Overridden from the base class
-        virtual void DoWorkCode(PicArray& in_data);
+        virtual void DoWorkCode(CoeffArray& in_data);
         // Ditto
-        virtual void DoWorkDecode(PicArray& out_data);
+        virtual void DoWorkDecode(CoeffArray& out_data);
 
-        virtual void CodeCoeffBlock(const CodeBlock& code_block , PicArray& in_data);
-        virtual void DecodeCoeffBlock(const CodeBlock& code_block , PicArray& out_data);
+        virtual void CodeCoeffBlock(const CodeBlock& code_block , CoeffArray& in_data);
+        virtual void DecodeCoeffBlock(const CodeBlock& code_block , CoeffArray& out_data);
 
-        void ResetAll();
+        //! A function for choosing the context for "follow bits"
+        inline int ChooseFollowContext( const int bin_number ) const;
 
-        // structure used to select the contexts used to encode / decode a symbol
-        typedef struct 
-        {
-            int firstContext;
-            int secondContext;
-            int endContext;
-        } ContextTriple;
+        //! A function for choosing the context for "information bits"
+        inline int ChooseInfoContext() const;
 
-        typedef enum ContextTripleAliases
-        {
-            Z_z_CTXT,
-            Z_nz_CTXT,
-            NZ_z_CTXT,
-            NZ_b_CTXT,
-            NZ_a_CTXT,
-            lastCTXTAlias
-        } ;
-
-        static const ContextTriple m_context_triples[lastCTXTAlias];
-
-        const ContextTriple& ChooseContexts() const;
-
-        int ChooseSignContext(const PicArray& data , const int xpos , const int ypos ) const;
+        //! A function for choosing the context for sign bits
+        inline int ChooseSignContext(const CoeffArray& data , const int xpos , const int ypos ) const;
 
         //! Private, bodyless copy constructor: class should not be copied
         BandCodec(const BandCodec& cpy);
@@ -141,29 +120,27 @@ namespace dirac
         BandCodec& operator=(const BandCodec& rhs);
 
     protected:
+              
+        //! Flag indicating whether the band comes from an intra frame
+        bool m_is_intra;
+    
         //! variables    
         int m_bnum;
 
         //! the subband being coded
         const Subband m_node;
     
-        //! size of the subband
-        int m_vol;
-    
-        //! the number of coefficients after which contexts are reset
-        int m_reset_coeff_num;
-    
-        //! count of the coefficients since the last context reset
-        int m_coeff_count;
-    
-        //! quantisation and inverse quantisation values
-        int m_qf, m_qfinv;
+        //! the quantisation index of the last codeblock
+        int m_last_qf_idx;
+            
+        //! quantisation value
+        int m_qf;
     
         //! reconstruction point
-        ValueType m_offset;
-    
-        //! sum of a neighbourhood of previously (de)coded values
-        ValueType m_nhood_sum;
+        CoeffType m_offset;
+
+        //! True if neighbours non-zero
+        bool m_nhood_nonzero;
     
         //! the parent subband
         Subband m_pnode;
@@ -177,8 +154,6 @@ namespace dirac
         //! True if the parent of a coeff is not zero
         bool m_parent_notzero;
     
-        //! used in selecting a context
-        ValueType m_cut_off_point;
     };
 
     //! A class specially for coding the LF subbands 
@@ -189,42 +164,31 @@ namespace dirac
     class LFBandCodec: public BandCodec
     {
     public:
-        //! Constructor for encoding
+        //! Constructor
         /*!
             Creates a LFBandCodec object to encode subband data.
-            \param    bits_out    the output for the encoded bits
+            \param    subband_byteio input/output for the encoded bits
             \param    number_of_contexts the number of contexts used in the encoding process
             \param    band_list    the set of all the subbands
-            \param     band_num    the number of the subband being coded 
+            \param    band_num    the number of the subband being coded 
+            \param    is_intra    Flag indicating whether the band comes from an intra frame
          */        
-        LFBandCodec(BasicOutputManager* bits_out,
+        LFBandCodec(SubbandByteIO* subband_byteio,
                     size_t number_of_contexts,
                     const SubbandList& band_list,
-                    int band_num)
-              : BandCodec(bits_out,number_of_contexts,band_list,band_num){}
-
-        //! Constructor for decoding
-        /*!
-            Creates a LFBandCodec object to decode subband data.
-            \param    bits_in        the input for the encoded bits
-            \param    number_of_contexts the number contexts used in the decoding process
-            \param    band_list    the set of all the subbands
-            \param     band_num    the number of the subband being decoded 
-         */
-        LFBandCodec(BitInputManager* bits_in,
-                    size_t number_of_contexts,
-                    const SubbandList& band_list,
-                    int band_num)
-          : BandCodec(bits_in,number_of_contexts,band_list,band_num){}
+                    int band_num,
+                    const bool is_intra)
+                    : BandCodec(subband_byteio,number_of_contexts,band_list,
+                                band_num,is_intra){}
 
     private:
         // Overridden from the base class
-        void DoWorkCode(PicArray& in_data);
+        void DoWorkCode(CoeffArray& in_data);
         // Ditto
-        void DoWorkDecode(PicArray& out_data);
+        void DoWorkDecode(CoeffArray& out_data);
 
-        void CodeCoeffBlock(const CodeBlock& code_block , PicArray& in_data);
-        void DecodeCoeffBlock(const CodeBlock& code_block , PicArray& out_data);
+        void CodeCoeffBlock(const CodeBlock& code_block , CoeffArray& in_data);
+        void DecodeCoeffBlock(const CodeBlock& code_block , CoeffArray& out_data);
 
         //! Private, bodyless copy constructor: class should not be copied
         LFBandCodec(const LFBandCodec& cpy);
@@ -246,36 +210,26 @@ namespace dirac
     class IntraDCBandCodec: public BandCodec
     {
     public:
-        //! Constructor for encoding
+        //! Constructor
         /*!
             Creates a IntraDCBandCodec object to encode subband data, based on parameters
-            \param    bits_out    the output for the encoded bits
+            \param    subband_byteio input/output for the encoded bits
             \param    number_of_contexts the number of contexts used in the encoding process
             \param    band_list    the set of all the subbands
          */
-        IntraDCBandCodec(BasicOutputManager* bits_out,
+        IntraDCBandCodec(SubbandByteIO* subband_byteio,
                          size_t number_of_contexts,
                          const SubbandList& band_list)
-          : BandCodec(bits_out,number_of_contexts,band_list,band_list.Length()){}
-
-        //! Constructor for decoding
-        /*!
-            Creates a LFBandCodec object to decode subband data, based on parameters
-            \param    bits_in        the input for the encoded bits
-            \param    number_of_contexts the number of contexts used in the decoding process
-            \param    band_list    the set of all the subbands
-         */    
-        IntraDCBandCodec(BitInputManager* bits_in,
-                         size_t number_of_contexts,
-                         const SubbandList& band_list)
-          : BandCodec(bits_in,number_of_contexts,band_list,band_list.Length()){}
+          : BandCodec(subband_byteio,number_of_contexts,band_list,
+                      band_list.Length(), true){}
 
+    
     private:
-        void DoWorkCode(PicArray& in_data);                    //overridden from the base class
-        void DoWorkDecode(PicArray& out_data); //ditto
+        void DoWorkCode(CoeffArray& in_data);                    //overridden from the base class
+        void DoWorkDecode(CoeffArray& out_data); //ditto
 
-        void CodeCoeffBlock(const CodeBlock& code_block , PicArray& in_data);
-        void DecodeCoeffBlock(const CodeBlock& code_block , PicArray& out_data);
+        void CodeCoeffBlock(const CodeBlock& code_block , CoeffArray& in_data);
+        void DecodeCoeffBlock(const CodeBlock& code_block , CoeffArray& out_data);
 
         //! Private, bodyless copy constructor: class should not be copied
         IntraDCBandCodec(const IntraDCBandCodec& cpy); 
@@ -284,10 +238,10 @@ namespace dirac
         IntraDCBandCodec& operator=(const IntraDCBandCodec& rhs);
 
         //! Prediction of a DC value from its previously coded neighbours
-        ValueType GetPrediction(const PicArray& data , const int xpos , const int ypos ) const;
+        CoeffType GetPrediction(const CoeffArray& data , const int xpos , const int ypos ) const;
 
     private:
-        PicArray m_dc_pred_res;
+        CoeffArray m_dc_pred_res;
     };
 
 
