@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: frame.cpp,v 1.11 2005/11/08 11:26:26 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: frame.cpp,v 1.20 2007/09/03 11:30:29 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -77,7 +77,7 @@ Frame::Frame( const Frame& cpy ):
     m_redo_upUdata(cpy.m_redo_upUdata),
     m_redo_upVdata(cpy.m_redo_upVdata)
 {
-    const ChromaFormat& cformat = m_fparams.CFormat();
+    //const ChromaFormat& cformat = m_fparams.CFormat();
 
     //delete data to be overwritten
     ClearData();
@@ -88,22 +88,17 @@ Frame::Frame( const Frame& cpy ):
         m_upY_data = new PicArray( *(cpy.m_upY_data) );
     }
 
-    if (cformat != Yonly)
+    m_U_data = new PicArray( *(cpy.m_U_data) );
+    m_V_data = new PicArray( *(cpy.m_V_data) );
+
+    if ( cpy.m_upU_data != 0 )
     {
-        m_U_data = new PicArray( *(cpy.m_U_data) );
-        m_V_data = new PicArray( *(cpy.m_V_data) );
-
-        if ( cpy.m_upU_data != 0 )
-        {
-            m_upU_data = new PicArray( *(cpy.m_upU_data) );
-        }
-        if ( cpy.m_upV_data != 0 )
-        {
-            m_upV_data = new PicArray( *(cpy.m_upV_data) );
-        }
-
+        m_upU_data = new PicArray( *(cpy.m_upU_data) );
+    }
+    if ( cpy.m_upV_data != 0 )
+    {
+        m_upV_data = new PicArray( *(cpy.m_upV_data) );
     }
-
 }
 
 
@@ -120,7 +115,7 @@ Frame& Frame::operator=(const Frame& rhs)
         m_redo_upYdata = rhs.m_redo_upYdata;
         m_redo_upUdata = rhs.m_redo_upUdata;
         m_redo_upVdata = rhs.m_redo_upVdata;
-        const ChromaFormat& cformat=m_fparams.CFormat();
+        //const ChromaFormat& cformat=m_fparams.CFormat();
 
         // Delete current data
         ClearData();
@@ -131,66 +126,61 @@ Frame& Frame::operator=(const Frame& rhs)
         if (rhs.m_upY_data != 0)
             m_upY_data = new PicArray( *(rhs.m_upY_data) );
 
-        if (cformat != Yonly)
-        {
-            m_U_data = new PicArray( *(rhs.m_U_data) );
-            if ( rhs.m_upU_data != 0 )
-                m_upU_data = new PicArray( *(rhs.m_upU_data) );
+        m_U_data = new PicArray( *(rhs.m_U_data) );
+        if ( rhs.m_upU_data != 0 )
+            m_upU_data = new PicArray( *(rhs.m_upU_data) );
 
-            m_V_data = new PicArray( *(rhs.m_V_data) );
-            if ( rhs.m_upV_data != 0 )
-                m_upV_data = new PicArray( *(rhs.m_upV_data) );
-        }
+        m_V_data = new PicArray( *(rhs.m_V_data) );
+        if ( rhs.m_upV_data != 0 )
+            m_upV_data = new PicArray( *(rhs.m_upV_data) );
     }
 
     return *this;
 
 }
 
+void Frame::CopyContents(Frame& out) const
+{
+    if ( &out != this)
+    {
+        out.m_redo_upYdata = true;
+        out.m_redo_upUdata = true;
+        out.m_redo_upVdata = true;
+        m_Y_data->CopyContents(*(out.m_Y_data));
+        m_U_data->CopyContents(*(out.m_U_data));
+        m_V_data->CopyContents(*(out.m_V_data));
+    }
+}
+
+void Frame::Fill(ValueType val)
+{
+    m_redo_upYdata = true;
+    m_redo_upUdata = true;
+    m_redo_upVdata = true;
+    m_Y_data->Fill(val);
+    m_U_data->Fill(val);
+    m_V_data->Fill(val);
+}
+
 //Other functions
 
 void Frame::Init()
 {
-    const ChromaFormat cformat=m_fparams.CFormat();
+    //const ChromaFormat cformat=m_fparams.CFormat();
 
     //first delete data if we need to
     ClearData();
 
-     m_Y_data=new PicArray( m_fparams.Yl() , m_fparams.Xl());
+     m_Y_data=new PicArray( m_fparams.DwtYl() , m_fparams.DwtXl());
      m_Y_data->SetCSort( Y_COMP );
 
-     if(cformat == format422) 
-     {
-         m_U_data = new PicArray( m_fparams.Yl() , m_fparams.Xl()/2 ); 
-         m_U_data->SetCSort( U_COMP );
+     m_U_data = new PicArray( m_fparams.DwtChromaYl() ,
+                              m_fparams.DwtChromaXl() ); 
+     m_U_data->SetCSort( U_COMP );
 
-         m_V_data = new PicArray( m_fparams.Yl() , m_fparams.Xl()/2 );
-         m_V_data->SetCSort( V_COMP );
-     }
-     else if (cformat == format420)
-     {
-         m_U_data = new PicArray( m_fparams.Yl()/2 , m_fparams.Xl()/2 );
-         m_U_data->SetCSort( U_COMP );
-
-         m_V_data = new PicArray( m_fparams.Yl()/2 , m_fparams.Xl()/2 ); 
-         m_V_data->SetCSort(V_COMP);
-     }
-     else if (cformat == format411)
-     {
-         m_U_data = new PicArray( m_fparams.Yl() , m_fparams.Xl()/4 );
-         m_U_data->SetCSort( U_COMP );
-
-         m_V_data = new PicArray( m_fparams.Yl() , m_fparams.Xl()/4 );
-         m_V_data->SetCSort( V_COMP );
-     }
-     else if (cformat==format444){
-         m_U_data = new PicArray( m_fparams.Yl() , m_fparams.Xl() ); 
-         m_U_data->SetCSort( U_COMP );
-
-         m_V_data = new PicArray( m_fparams.Yl() , m_fparams.Xl() );
-         m_V_data->SetCSort( V_COMP );
-    }
-    //(other formats all assumed to be Yonly
+     m_V_data = new PicArray( m_fparams.DwtChromaYl() ,
+                              m_fparams.DwtChromaXl() );
+     m_V_data->SetCSort( V_COMP );
 }
 
 PicArray& Frame::Data(CompSort cs)
@@ -217,8 +207,11 @@ PicArray& Frame::UpYdata()
     {//we have to do the upconversion
 
         if (m_upY_data == 0)
-            m_upY_data = new PicArray( 2*m_Y_data->LengthY() , 2*m_Y_data->LengthX() );
-        UpConverter myupconv;
+            m_upY_data = new PicArray( 2*m_Y_data->LengthY(),
+                                       2*m_Y_data->LengthX() );
+        UpConverter myupconv(-(1 << (m_fparams.LumaDepth()-1)), 
+                             (1 << (m_fparams.LumaDepth()-1))-1,
+                             m_fparams.OrigXl(), m_fparams.OrigYl());
         myupconv.DoUpConverter( *m_Y_data , *m_upY_data );
 
         m_redo_upYdata = false;
@@ -235,8 +228,13 @@ PicArray& Frame::UpUdata()
     {//we have to do the upconversion
 
         if (m_upU_data ==0)
-            m_upU_data = new PicArray(2*m_U_data->LengthY() , 2*m_U_data->LengthX());
-        UpConverter myupconv;
+            m_upU_data = new PicArray(2*m_U_data->LengthY() ,
+                                      2*m_U_data->LengthX());
+        UpConverter myupconv(-(1 << (m_fparams.ChromaDepth()-1)), 
+                             (1 << (m_fparams.ChromaDepth()-1))-1,
+                                 m_fparams.OrigChromaXl(),
+                                 m_fparams.OrigChromaYl());
+
         myupconv.DoUpConverter( *m_U_data , *m_upU_data );
         m_redo_upUdata = false;
         return *m_upU_data;
@@ -252,8 +250,12 @@ PicArray& Frame::UpVdata()
     {//we have to do the upconversion
    
            if (m_upV_data ==0)
-            m_upV_data = new PicArray( 2*m_V_data->LengthY() , 2*m_V_data->LengthX() );
-        UpConverter myupconv;
+            m_upV_data = new PicArray( 2*m_V_data->LengthY(),
+                                       2*m_V_data->LengthX() );
+        UpConverter myupconv(-(1 << (m_fparams.ChromaDepth()-1)), 
+                             (1 << (m_fparams.ChromaDepth()-1))-1,
+                                 m_fparams.OrigChromaXl(),
+                                 m_fparams.OrigChromaYl());
         myupconv.DoUpConverter( *m_V_data , *m_upV_data );
         m_redo_upVdata = false;
 
@@ -284,9 +286,12 @@ const PicArray& Frame::UpYdata() const
         //declared mutable.
 
         if (m_upY_data == 0)
-            m_upY_data = new PicArray( 2*m_Y_data->LengthY() , 2*m_Y_data->LengthX() );
+            m_upY_data = new PicArray( 2*m_Y_data->LengthY(),
+                                       2*m_Y_data->LengthX() );
 
-        UpConverter myupconv;
+        UpConverter myupconv(-(1 << (m_fparams.LumaDepth()-1)), 
+                             (1 << (m_fparams.LumaDepth()-1))-1,
+                             m_fparams.OrigXl(), m_fparams.OrigYl());
         myupconv.DoUpConverter( *m_Y_data , *m_upY_data );
 
         m_redo_upYdata = false;
@@ -307,9 +312,13 @@ const PicArray& Frame::UpUdata() const
         //declared mutable.
 
         if (m_upU_data == 0)
-            m_upU_data = new PicArray( 2*m_U_data->LengthY() , 2*m_U_data->LengthX() );
+            m_upU_data = new PicArray( 2*m_U_data->LengthY(),
+                                       2*m_U_data->LengthX() );
 
-        UpConverter myupconv;
+        UpConverter myupconv(-(1 << (m_fparams.ChromaDepth()-1)), 
+                             (1 << (m_fparams.ChromaDepth()-1))-1,
+                                 m_fparams.OrigChromaXl(),
+                                 m_fparams.OrigChromaYl());
         myupconv.DoUpConverter( *m_U_data , *m_upU_data );
         m_redo_upUdata = false;
 
@@ -330,9 +339,13 @@ const PicArray& Frame::UpVdata() const
         //declared mutable.
  
         if (m_upV_data == 0)
-            m_upV_data = new PicArray( 2*m_V_data->LengthY() , 2*m_V_data->LengthX() );
+            m_upV_data = new PicArray( 2*m_V_data->LengthY() ,
+                                       2*m_V_data->LengthX() );
 
-        UpConverter myupconv;
+        UpConverter myupconv(-(1 << (m_fparams.ChromaDepth()-1)), 
+                             (1 << (m_fparams.ChromaDepth()-1))-1,
+                                 m_fparams.OrigChromaXl(),
+                                 m_fparams.OrigChromaYl());
         myupconv.DoUpConverter( *m_V_data , *m_upV_data );
         m_redo_upVdata = false;
 
@@ -351,21 +364,33 @@ const PicArray& Frame::UpData(CompSort cs) const
         return UpYdata();
 }    
 
-void Frame::ClipComponent(PicArray& pic_data)
+void Frame::ClipComponent(PicArray& pic_data, CompSort cs) const
 {
     ValueType *pic = &(pic_data[pic_data.FirstY()][pic_data.FirstX()]);
     int count = pic_data.LengthY() * pic_data.LengthX();
 
+    ValueType min_val;
+    ValueType max_val;
+    
+    min_val = (cs == Y_COMP) ?
+              -(1 << (m_fparams.LumaDepth()-1) ) :
+              -(1 << (m_fparams.ChromaDepth()-1) );
+
+    max_val = (cs == Y_COMP) ?
+              (1 << (m_fparams.LumaDepth()-1) )-1 :
+              (1 << (m_fparams.ChromaDepth()-1) )-1;
+
 #if defined (HAVE_MMX)
     {
         int qcount = count >> 2;
         count = count & 3;
     
-        __m64 pack_usmax = _mm_set_pi16 (0xffff, 0xffff, 0xffff, 0xffff);
-           //__m64 pack_smax = _mm_set_pi16 (0x7fff, 0x7fff, 0x7fff, 0x7fff);
-           __m64 pack_smin = _mm_set_pi16 (0x8000, 0x8000, 0x8000, 0x8000);
-           __m64 high_val = _mm_set_pi16 (PIXEL_VALUE_MAX, PIXEL_VALUE_MAX, PIXEL_VALUE_MAX, PIXEL_VALUE_MAX);
-           __m64 lo_val = _mm_set_pi16 (PIXEL_VALUE_MIN, PIXEL_VALUE_MIN, PIXEL_VALUE_MIN, PIXEL_VALUE_MIN);
+        //__m64 pack_usmax = _mm_set_pi16 (0xffff, 0xffff, 0xffff, 0xffff);
+        //__m64 pack_smin = _mm_set_pi16 (0x8000, 0x8000, 0x8000, 0x8000);
+        __m64 pack_usmax = _mm_set_pi16 (-1, -1, -1, -1);
+        __m64 pack_smin = _mm_set_pi16 (-32768, -32768, -32768, -32768);
+        __m64 high_val = _mm_set_pi16 (max_val, max_val, max_val, max_val);
+        __m64 lo_val = _mm_set_pi16 (min_val, min_val, min_val, min_val);
     
         __m64 clip_max = _mm_add_pi16 (pack_smin, high_val);
         __m64 clip_min = _mm_add_pi16 (pack_smin, lo_val);
@@ -385,9 +410,7 @@ void Frame::ClipComponent(PicArray& pic_data)
         //Mop up remaining pixels
             while( count-- )
         {
-            *pic = std::max( 
-                    ValueType( PIXEL_VALUE_MIN ), 
-                    std::min( ValueType( PIXEL_VALUE_MAX ), *pic ) 
+            *pic = std::max( min_val, std::min( max_val , *pic ) 
                     );
             pic++;
         }
@@ -400,8 +423,7 @@ void Frame::ClipComponent(PicArray& pic_data)
     // NOTE: depending on a contigous chunk of memory being allocated
     while (count--)
     {
-        *pic = std::max( (ValueType) PIXEL_VALUE_MIN,  
-        std::min( ValueType( PIXEL_VALUE_MAX ), *pic ));
+        *pic = std::max( min_val, std::min( max_val, *pic ));
         pic++;
     }
 }
@@ -410,13 +432,24 @@ void Frame::Clip()
 {
     //just clips the straight picture data, not the upconverted data
 
-    ClipComponent( *m_Y_data );
+    ClipComponent( *m_Y_data, Y_COMP );
 
-    if (m_fparams.CFormat() != Yonly)
-    {
-        ClipComponent( *m_U_data );
-        ClipComponent( *m_V_data );    
-    }    
+    ClipComponent( *m_U_data, U_COMP );
+    ClipComponent( *m_V_data, V_COMP );    
+}
+
+void Frame::ClipUpData()
+{
+    //just clips the upconverted data
+
+    if (m_upY_data)
+        ClipComponent( *m_upY_data, Y_COMP );
+
+    if (m_upU_data)
+        ClipComponent( *m_upU_data, U_COMP );
+    
+    if (m_upV_data)
+        ClipComponent( *m_upV_data, V_COMP );    
 }
 
 void Frame::ClearData()
@@ -466,7 +499,8 @@ void Frame::ReconfigFrame(const FrameParams &fp )
     m_redo_upYdata = m_redo_upUdata = m_redo_upVdata = true;
 
     // HAve frame dimensions  or Chroma format changed ?
-    if (m_fparams.Xl() == old_fp.Xl() && m_fparams.Yl() == old_fp.Yl() &&
+    if (m_fparams.DwtXl() == old_fp.DwtXl() && 
+        m_fparams.DwtYl() == old_fp.DwtYl() &&
         m_fparams.CFormat() == old_fp.CFormat())
         return;
 
