@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: frame.h,v 1.9 2005/11/08 11:26:26 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: frame.h,v 1.13 2007/09/03 11:30:29 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -67,13 +67,32 @@ namespace dirac
         //! Assignment =. Private as not currently used [may want to implement reference counting later.]
         Frame& operator=( const Frame& rhs );
 
+        //! Frame Copy
+        /*!
+            Copy contents of frame into the output frame passed to it 
+            retaining the frame dimensions of the output frame.
+        */
+        void CopyContents(Frame& out ) const;
+
+        //! Frame Fill
+        /*!
+            Initialise contents of frame with value provided
+        */
+        void Fill(ValueType val );
+
         //gets and sets
         //! Gets the frame parameters
-        const FrameParams& GetFparams() const {return m_fparams;}
+        FrameParams& GetFparams() const  {return m_fparams;}
 
         //! Sets the frame sort
         void SetFrameSort( const FrameSort fs ){m_fparams.SetFSort( fs ); }
 
+        //! Sets the frame type
+        void SetFrameType( const FrameType ftype ){m_fparams.SetFrameType( ftype ); }
+
+        //! Sets the frame type
+        void SetReferenceType( const ReferenceType rtype ){m_fparams.SetReferenceType( rtype ); }
+
         //! Reconfigures the the framend to the new parameters. 
         void ReconfigFrame( const FrameParams &fp );
 
@@ -127,12 +146,18 @@ namespace dirac
 
         //! Clip the data to prevent overshoot
         /*!
-            Clips the data to lie between 0 and 1020 (4*255) in 10-bit form to prevent overshoot/wraparound.
+            Clips the data to lie between 0 and (1<<video_depth)-1 
          */
         void Clip();
 
+        //! Clip the upconverted data to prevent overshoot
+        /*!
+            Clips the upconverted data to lie between 0 and (1<<video_depth)-1 
+         */
+        void ClipUpData();
+
     private:
-        FrameParams m_fparams;
+        mutable FrameParams m_fparams;
         PicArray* m_Y_data;//the 
         PicArray* m_U_data;//component
         PicArray* m_V_data;//data
@@ -147,7 +172,7 @@ namespace dirac
         void ClearData();
 
         //! Clip an individual component
-        void ClipComponent(PicArray& pic_data);
+        void ClipComponent(PicArray& pic_data, CompSort cs) const;
 
         //! Flag that upconversion needs to be re-done
         mutable bool m_redo_upYdata;
