@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: motion.cpp,v 1.16 2005/08/12 09:26:16 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: motion.cpp,v 1.21 2007/07/30 18:40:01 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -20,7 +20,9 @@
 * Portions created by the Initial Developer are Copyright (C) 2004.
 * All Rights Reserved.
 *
-* Contributor(s): Thomas Davies (Original Author), Chris Bowley
+* Contributor(s): Thomas Davies (Original Author),
+*                 Chris Bowley,
+*                 Tim Borer
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -57,8 +59,8 @@ MvData::MvData( const int xnumMB, const int ynumMB ,
     m_modes( ynumblocks , xnumblocks ),
     m_dc( 3 ),
     m_mb_split( ynumMB , xnumMB ),
-    m_mb_common( ynumMB , xnumMB ),
-    m_gm_params( Range(1 , num_refs) )
+    m_gm_params( Range(1 , num_refs) ),
+    m_num_refs(num_refs)
 {
 
     InitMvData();
@@ -70,8 +72,8 @@ MvData::MvData( const int xnumMB , const int ynumMB , const int num_refs ):
     m_modes( 4*ynumMB , 4*xnumMB ),
     m_dc( 3 ),
     m_mb_split( ynumMB , xnumMB ),
-    m_mb_common( ynumMB , xnumMB ),
-    m_gm_params( Range(1 , num_refs) )
+    m_gm_params( Range(1 , num_refs) ),
+    m_num_refs(num_refs)
 {
     InitMvData();
 }
@@ -91,7 +93,7 @@ void MvData::InitMvData()
 
      // Create the arrays of dc values
      for ( int i=0 ; i<3 ; ++i )
-         m_dc[i] = new TwoDArray<ValueType>( Mode().LengthY() , Mode().LengthX() );
+         m_dc[i] = new TwoDArray<ValueType>( Mode().LengthY() , Mode().LengthX() , 0);
 }
 
 MvData::~MvData()
@@ -118,7 +120,7 @@ MEData::MEData(const int xnumMB , const int ynumMB ,
                 const int xnumblocks , const int ynumblocks , const int num_refs ):
      MvData( xnumMB , ynumMB , xnumblocks , ynumblocks , num_refs ),
      m_pred_costs( Range( 1 , num_refs ) ),
-     m_intra_costs( ynumblocks , xnumblocks ),
+     m_intra_costs( ynumblocks , xnumblocks, 0 ),
      m_bipred_costs( ynumblocks , xnumblocks ),
      m_MB_costs( ynumMB , xnumMB ),
      m_lambda_map( ynumblocks , xnumblocks ),
@@ -130,7 +132,7 @@ MEData::MEData(const int xnumMB , const int ynumMB ,
 MEData::MEData( const int xnumMB , const int ynumMB ,  const int num_refs ):
      MvData( xnumMB , ynumMB , num_refs ),
      m_pred_costs( Range( 1 , num_refs ) ),
-     m_intra_costs( 4*ynumMB , 4*xnumMB ),
+     m_intra_costs( 4*ynumMB , 4*xnumMB, 0 ),
      m_bipred_costs( 4*ynumMB , 4*xnumMB ),
      m_MB_costs( ynumMB , xnumMB ),
      m_lambda_map( 4*ynumMB , 4*xnumMB ),
@@ -415,7 +417,6 @@ istream &operator>> (istream & stream, MEData & me_data)
     
     // input reference-independent information
     stream >> me_data.MBSplit();
-    stream >> me_data.MBCommonMode();
     stream >> me_data.MBCosts();
     stream >> me_data.Mode();
     stream >> me_data.IntraCosts();
@@ -452,7 +453,6 @@ ostream &operator<< (ostream & stream, MEData & me_data)
 {
     // output reference-independent information
     stream << endl << endl << me_data.MBSplit();
-    stream << endl << me_data.MBCommonMode();
     stream << endl << me_data.MBCosts();
     stream << endl << me_data.Mode();
     stream << endl << me_data.IntraCosts() << endl;
@@ -485,6 +485,20 @@ ostream &operator<< (ostream & stream, MEData & me_data)
     return stream;
 }
 
+int Median( const int val1, const int val2, const int val3)
+{
+    int tmp;
+
+    tmp=val1;
+    tmp+=val2;
+    tmp+=val3;
+
+    tmp -= std::max( std::max( val1 , val2 ) , val3 );
+    tmp -= std::min( std::min( val1 , val2 ) , val3 );
+    
+    return tmp;
+}
+
 MVector MvMedian(const MVector& mv1,const MVector& mv2,const MVector& mv3) {
     //takes median of each vector component    
     MVector tmp_mv;
@@ -506,89 +520,199 @@ MVector MvMedian(const MVector& mv1,const MVector& mv2,const MVector& mv3) {
     return tmp_mv;
 }
 
-MVector MvMedian(const std::vector<MVector>& vect_list){
-    //more general median. Takes the median of each vector component    
+int Median(const std::vector<int>& val_list)
+{   
+    // take the median of up to 4 elements
 
-    MVector median;
-    int num_vals=int(vect_list.size());
-    if (num_vals>0)    {
-        int pos=0;
-        std::vector<int> ordered_vals(vect_list.size());
-        //do x first
-        ordered_vals[0]=vect_list[0].x;        
-        for (int I=1;I<num_vals;++I){
-            for (int K=0;K<I;++K){
-                if (vect_list[I].x<ordered_vals[K]){
-                    pos=K;
-                    break;
-                }
-                else
-                    pos=K+1;
-            }//K
-            if (pos==I)
-                ordered_vals[I]=vect_list[I].x;
-            else{
-                for (int K=I-1;K>=pos;--K){
-                    ordered_vals[K+1]=ordered_vals[K];
-                }
-                ordered_vals[pos]=vect_list[I].x;
+    switch (val_list.size() )
+    {
+        case 1 :
+
+            return val_list[0];
+        case 2 : // return the mean
+            return ( ( val_list[0] + val_list[1] + 1 )>>1 );
+        case 3 :
+            return Median(val_list[0], val_list[1], val_list[2] );
+        case 4 :
+            {
+            int med_val(0);
+            int max_val(val_list[0]);
+            int min_val(val_list[0]);
+
+            for (int i=0; i<4; ++i )
+            {
+                med_val += val_list[i];
+                max_val = std::max( max_val , val_list[i] );
+                min_val = std::min( min_val , val_list[i] );
+
+            }// i
+         
+            med_val -= ( max_val + min_val );
+         
+            return ( (med_val + 1)>>1 );
             }
-        }//I
-        if (vect_list.size()%2!=0)
-            median.x=ordered_vals[(num_vals-1)/2];
-        else
-            median.x=(ordered_vals[(num_vals/2)-1]+ordered_vals[num_vals/2])/2;
-
-        //now do y
-        ordered_vals[0]=vect_list[0].y;        
-        for (int I=1;I<num_vals;++I){
-            for (int K=0;K<I;++K){
-                if (vect_list[I].y<ordered_vals[K]){
-                    pos=K;
-                    break;
-                }
-                else
-                    pos=K+1;
-            }//K
-            if (pos==I)
-                ordered_vals[I]=vect_list[I].y;
-            else{
-                for (int K=I-1;K>=pos;--K){
-                    ordered_vals[K+1]=ordered_vals[K];
-                }
-                ordered_vals[pos]=vect_list[I].y;
-            }
-        }//I
-        if (num_vals%2!=0)
-            median.y=ordered_vals[(num_vals-1)/2];
-        else
-            median.y=(ordered_vals[(num_vals/2)-1]+ordered_vals[num_vals/2])/2;        
-
-    }
-    else{
-        median.x=0;
-        median.y=0;
+        default :
+            return 0;
     }
 
-    return median;
 }
 
-//! Return the mean of two motion vectors
-MVector MvMean(MVector& mv1,MVector& mv2) {
-    //takes median of each vector component    
+MVector MvMedian(const std::vector<MVector>& vect_list){
+    //median of 0-4 vectors
+    
+    if ( vect_list.size() == 0 )
+        return 0;
+    else if ( vect_list.size() == 1 )
+        return vect_list[0];
+    else if ( vect_list.size() == 2 )
+        return MvMean( vect_list[0], vect_list[1] );
+    else if ( vect_list.size() == 3 )
+        return MvMedian(vect_list[0], vect_list[1], vect_list[2] );
+    else if ( vect_list.size() == 4 )
+    {
+         MVector tmp_mv(0);
+         MVector max_mv(vect_list[0]);
+         MVector min_mv(vect_list[0]);
+         for (int i=0; i<4; ++i )
+         {
+             tmp_mv.x += vect_list[i].x;
+             max_mv.x=std::max(max_mv.x, vect_list[i].x);
+             min_mv.x=std::min(min_mv.x, vect_list[i].x);
+
+             tmp_mv.y += vect_list[i].y;
+             max_mv.y=std::max(max_mv.y, vect_list[i].y);
+             min_mv.y=std::min(min_mv.y, vect_list[i].y);            
+
+         }// i
+         
+         tmp_mv.x -= (max_mv.x+min_mv.x);
+         tmp_mv.y -= (max_mv.y+min_mv.y);
+         
+         tmp_mv.x = (tmp_mv.x+1)>>1;
+         tmp_mv.y = (tmp_mv.y+1)>>1;
+
+         return tmp_mv;    
+             
+    }
+    else
+    {        
+        MVector median;
+        int num_vals=int(vect_list.size());
+        if (num_vals>0)    {
+            int pos=0;
+            std::vector<int> ordered_vals(vect_list.size());
+            //do x first
+            ordered_vals[0]=vect_list[0].x;        
+            for (int I=1;I<num_vals;++I){
+                for (int K=0;K<I;++K){
+                    if (vect_list[I].x<ordered_vals[K]){
+                        pos=K;
+                        break;
+                    }
+                    else
+                        pos=K+1;
+                }//K
+                if (pos==I)
+                    ordered_vals[I]=vect_list[I].x;
+                else{
+                    for (int K=I-1;K>=pos;--K){
+                        ordered_vals[K+1]=ordered_vals[K];
+                    }
+                    ordered_vals[pos]=vect_list[I].x;
+                }
+            }//I
+            if (vect_list.size()%2!=0)
+                median.x=ordered_vals[(num_vals-1)/2];
+            else
+                median.x=(ordered_vals[(num_vals/2)-1]+ordered_vals[num_vals/2]+1)>>1;
+
+            //now do y
+            ordered_vals[0]=vect_list[0].y;        
+            for (int I=1;I<num_vals;++I){
+                for (int K=0;K<I;++K){
+                    if (vect_list[I].y<ordered_vals[K]){
+                        pos=K;
+                        break;
+                    }
+                    else
+                        pos=K+1;
+                }//K
+                if (pos==I)
+                    ordered_vals[I]=vect_list[I].y;
+                else{
+                    for (int K=I-1;K>=pos;--K){
+                        ordered_vals[K+1]=ordered_vals[K];
+                    }
+                    ordered_vals[pos]=vect_list[I].y;
+                }
+            }//I
+            if (num_vals%2!=0)
+                median.y=ordered_vals[(num_vals-1)/2];
+            else
+                median.y=(ordered_vals[(num_vals/2)-1]+ordered_vals[num_vals/2]+1)>>1;        
+
+        }
+        else{
+            median.x=0;
+            median.y=0;
+        }
+
+        return median;
+    }
+
+}
+
+
+//! Return the unbiased mean of two motion vectors
+MVector MvMean(const MVector& mv1, const MVector& mv2)
+{
+    //takes mean of each vector component    
     MVector tmp_mv;
 
-    tmp_mv.x=mv1.x;
-    tmp_mv.x+=mv2.x;
-    tmp_mv.x/=2;
+    tmp_mv.x = mv1.x;
+    tmp_mv.x += mv2.x+1;
+    tmp_mv.x >>= 1;
 
-    tmp_mv.y=mv1.y;
-    tmp_mv.y+=mv2.y;
-    tmp_mv.y/=2;
+    tmp_mv.y = mv1.y;
+    tmp_mv.y += mv2.y+1;
+    tmp_mv.y >>= 1;
 
     return tmp_mv;
 }
 
+//! Return the mean of a set of unsigned integer values
+unsigned int GetUMean(std::vector<unsigned int>& values)
+{
+    unsigned int sum=0;
+    for (unsigned int I=0;I<values.size();++I)
+        sum+=values[I];
 
+    sum+=(values.size()>>1);
+    sum/=values.size();
+
+    return sum;
+}
+
+//! Return the mean of a set of signed integer values
+int GetSMean(std::vector<int>& values)
+{
+    int sum=0;
+    for (unsigned int I=0;I<values.size();++I)
+        sum+=values[I];
+    if ( sum>0 )
+    {
+        sum+=(values.size()>>1);
+        sum/=values.size();
+    }
+    else
+    {
+        sum = -sum;
+        sum+=(values.size()>>1);
+        sum/=values.size();
+        sum = -sum;
+    }
+
+    return sum;
+}
 
 } // namespace dirac
