@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: motion.h,v 1.15 2005/08/10 07:50:53 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: motion.h,v 1.21 2007/04/11 08:08:49 tjdwave Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -20,7 +20,9 @@
 * Portions created by the Initial Developer are Copyright (C) 2004.
 * All Rights Reserved.
 *
-* Contributor(s): Thomas Davies (Original Author), Chris Bowley
+* Contributor(s): Thomas Davies (Original Author),
+*                 Chris Bowley,
+*                 Tim Borer
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -48,6 +50,9 @@ namespace dirac
 
     //classes
 
+    //! Horizontal or vertical
+    enum MvElement { HORIZONTAL , VERTICAL };
+
     //! Motion vector class - just a pair
     template <class T>
     class MotionVector
@@ -78,6 +83,12 @@ namespace dirac
 
         //! Bitshift of each component
         inline MotionVector<T> operator>>(const int argument) const;
+        
+        //! Array-style element access
+        T& operator[](const int pos){return ( ( pos==0) ? x : y );}
+
+        //! Array-style element access.
+        const T& operator[](const int pos) const {return ( ( pos==0) ? x : y );}
 
 
         //! x and y components 
@@ -217,8 +228,8 @@ namespace dirac
             \param  ynumblocks  the number of blocks vertically
             \param  num_refs  the number of references being used for the frame
         */
-        MvData( const int xnumMB, int ynumMB , 
-                const int xnumblocks, int ynumblocks ,  const int num_refs = 2);
+        MvData( const int xnumMB, const int ynumMB , 
+                const int xnumblocks, const int ynumblocks ,  const int num_refs);
 
         //! Constructor
         /*! 
@@ -227,7 +238,7 @@ namespace dirac
             \param  ynumMB  the number of MBs vertically
             \param  num_refs  the number of references being used for the frame
         */
-        MvData( const int xnumMB, int ynumMB ,  const int num_refs = 2);
+        MvData( const int xnumMB, const int ynumMB ,  const int num_refs);
 
         //! Destructor
         ~MvData();
@@ -265,18 +276,15 @@ namespace dirac
         //! Get the MB split level
         const TwoDArray<int>& MBSplit() const{return m_mb_split;}
 
-        //! Get the MB common mode parameters
-        TwoDArray<bool>& MBCommonMode(){return m_mb_common;}
-
-        //! Get the MB common mode parameters
-        const TwoDArray<bool>& MBCommonMode() const{return m_mb_common;}
-
         //! Get the global motion model parameters
         OneDArray<float>& GlobalMotionParameters(const int ref_id) { return *( m_gm_params[ref_id] ); }
 
         //! Get the global motion model parameters
         const OneDArray<float>& GlobalMotionParameters(const int ref_id) const { return *( m_gm_params[ref_id] ); }
 
+        //! Return the number of reference frames
+        const unsigned int NumRefs()const {return m_num_refs;}
+
     private:
         // Initialises the arrays of data
         void InitMvData();
@@ -296,11 +304,11 @@ namespace dirac
         // The MB split levels
         TwoDArray<int> m_mb_split;
 
-        // The MB common mode indicators 
-        TwoDArray<bool> m_mb_common;
-
         // Global motion model parameters
         OneDArray< OneDArray<float>* > m_gm_params;
+
+        // Number of reference frames
+        const unsigned int m_num_refs;
     };
 
     //! Class for all the motion estimation data
@@ -410,15 +418,22 @@ namespace dirac
     };
 
     //motion estimation and coding stuff
+    
+    //! Return the median of 3 integers
+    int Median( const int val1, const int val2, const int val3);
 
     //! Return the median of three motion vectors 
     MVector MvMedian(const MVector& mv1,const MVector& mv2,const MVector& mv3);
 
-    //! Return the median of a set of motion vectors 
+
+    //! Return the median of a set of integers 
+    int Median(const std::vector<int>& val_list);
+
+    //! Return the median of a set of (up to 4) motion vectors 
     MVector MvMedian(const std::vector<MVector>& vect_list);
 
     //! Return the mean of two motion vectors
-    MVector MvMean(MVector& mv1,MVector& mv2);
+    MVector MvMean(const MVector& mv1, const MVector& mv2);
 
     //! Return the squared length of a motion vector
     inline int Norm2(const MVector& mv){//L^2 norm of a motion vector
@@ -430,24 +445,11 @@ namespace dirac
         return abs(mv.x)+abs(mv.y);
     }
 
-    //! Return the mean of a set of integer values
-    inline int GetMean(std::vector<int>& values){
-        int sum=0;
-        for (unsigned int I=0;I<values.size();++I)
-            sum+=values[I];
-        sum/=int(values.size());
-        return sum;
-    }
-
     //! Return the mean of a set of unsigned integer values
-    inline unsigned int GetMean(std::vector<unsigned int>& values){
-        int sum=0;
-        for (unsigned int I=0;I<values.size();++I)
-            sum+=values[I];
-        sum+=(values.size()>>1);
-        sum/=values.size();
-        return sum;
-    }
+    unsigned int GetUMean(std::vector<unsigned int>& values);
+    
+    //! Return the mean of a set of signed integer values
+    int GetSMean(std::vector<int>& values);
 
 } // namespace dirac
 
