@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: upconvert.h,v 1.9 2005/07/08 09:26:51 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: upconvert.h,v 1.13 2007/11/22 15:19:28 tjdwave Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -56,15 +56,15 @@ namespace dirac
     public:
 
         //! Constructor
-        UpConverter();
+        UpConverter(int min_val, int max_val, int orig_xlen, int orig_ylen);
 
         //! Destructor
-        ~UpConverter();
+        ~UpConverter() {};
 
         //! Upconvert the picture data
         /*!
             Upconvert the picture data, where the parameters are
-            \param    pic_data    is the original data
+            \param    pic_data   is the original data
             \param    up_data    is the upconverted data
          */
         void DoUpConverter(const PicArray& pic_data, PicArray& up_data);
@@ -77,23 +77,20 @@ namespace dirac
         UpConverter& operator=(const UpConverter& rhs);
 
         //! Applies the filter to a row and its successor 
-        void RowLoop(PicArray& up_data, const int row_num );
+        void RowLoop(PicArray& up_data, const int row_num,
+        const int filter_size, const int filter_shift, const short taps[4] );
 
     private:
         //Variable to keep the loops in check
         int m_width_old, m_height_old;
         int m_width_new, m_height_new;
 
-        //Define first set of filter parameters
-        static const int m_filter_size = 5;
-        static const int m_filter_shift = 8;
 
-        static const int m_tap0 = 167;
-        static const int m_tap1 = -56; 
-        static const int m_tap2 = 25;
-        static const int m_tap3 = -11; 
-        static const int m_tap4 = 3;
+        const int m_min_val;
+        const int m_max_val;
 
+        const int m_orig_xl;
+        const int m_orig_yl;
     };
 
 } // namespace dirac
