@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: wavelet_utils.h,v 1.17 2005/12/01 14:25:00 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: wavelet_utils.h,v 1.28 2007/12/12 14:01:51 tjdwave Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -20,7 +20,9 @@
 * Portions created by the Initial Developer are Copyright (C) 2004.
 * All Rights Reserved.
 *
-* Contributor(s): Thomas Davies (Original Author), Scott R Ladd
+* Contributor(s): Thomas Davies (Original Author),
+*                 Scott R Ladd
+*                 Anuradha Suraparaju
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -70,9 +72,9 @@ namespace dirac
         /*
             Initialise the code block
             \param    xstart  the x-coord of the first coefficient in the block
-            \param    xend    one past the last coefficient, horizontally    
+            \param    xend    one past the last coefficient, horizontally
             \param    ystart  the y-coord of the first coefficient in the block
-            \param    yend    one past the last coefficient, vertically    
+            \param    yend    one past the last coefficient, vertically
         */
         CodeBlock( const int xstart , const int ystart , const int xend , const int yend);
 
@@ -81,7 +83,7 @@ namespace dirac
 
         //! Returns the vertical start of the block
         int Ystart() const { return m_ystart; }
-    
+
         //! Returns one past the last coefficient coord, horizontally
         int Xend() const { return m_xend; }
 
@@ -97,13 +99,10 @@ namespace dirac
         //! Returns the quantisation index associated to the code block
         int QIndex() const{ return m_qindex; }
 
-        //! Returns the weight associated to the code block
-        float Wt() const { return m_wt; }
-
         //! Returns true if the code-block is skipped, false if not
         bool Skipped() const { return m_skipped; }
 
-        //! Sets the quantisation index 
+        //! Sets the quantisation index
         void SetQIndex( const int qindex ){ m_qindex = qindex; }
 
         //! Sets whether the code block is skipped or not
@@ -115,16 +114,12 @@ namespace dirac
         /*
             Initialise the code block
             \param    xstart  the x-coord of the first coefficient in the block
-            \param    xend    one past the last coefficient, horizontally    
+            \param    xend    one past the last coefficient, horizontally
             \param    ystart  the y-coord of the first coefficient in the block
-            \param    yend    one past the last coefficient, vertically    
+            \param    yend    one past the last coefficient, vertically
         */
         void Init( const int xstart , const int ystart , const int xend , const int yend );
 
-        //! Sets the perceptual weight (can only be done by friend class, eg Subband 
-        void SetWt( const float w ){ m_wt = w; }
-
-
     private:
 
         int m_xstart;
@@ -135,7 +130,6 @@ namespace dirac
         int m_yl;
 
         int m_qindex;
-        float m_wt;
 
         bool m_skipped;
     };
@@ -152,12 +146,12 @@ namespace dirac
         //! Constructor
         /*!
             The constructor parameters are
-            \param    xpos    the xposition of the subband when packed into a 
+            \param    xpos    the xposition of the subband when packed into a
                               big array with all the others
             \param    ypos    the xposition of the subband
             \param    xlen    the width of the subband
             \param    ylen    the height of the subband
-         */    
+         */
         Subband(int xpos, int ypos, int xlen, int ylen);
 
         //! Constructor
@@ -169,7 +163,7 @@ namespace dirac
             \param    xlen    the width of the subband
             \param    ylen    the height of the subband
             \param    d        the depth of the subband in the wavelet transform
-         */    
+         */
         Subband(int xpos, int ypos, int xlen, int ylen, int d);
 
         //! Destructor
@@ -179,67 +173,67 @@ namespace dirac
 
         //! Return the width of the subband
         int Xl() const {return m_xl;}
-    
+
         //! Return the horizontal position of the subband
         int Xp() const {return m_xp;}
-    
+
         //! Return the height of the subband
         int Yl() const {return m_yl;}
-    
+
         //! Return the vertical position of the subband
         int Yp() const {return m_yp;}
-    
+
         //! Return the index of the maximum bit of the largest coefficient
         int Max() const {return m_max_bit;}
-    
+
         //! Return the subband perceptual weight
         double Wt() const {return m_wt;}
-    
+
         //! Return the depth of the subband in the transform
         int Depth() const {return m_depth;}
-    
+
         //! Return the scale of the subband, viewed as a subsampled version of the picture
         int Scale() const {return ( 1<<m_depth );}
-    
+
         //! Return a quantisation factor
         int QIndex() const {return m_qindex;}
 
         //! Return a flag indicating whether we have separate quantisers for each code block
-        bool UsingMultiQuants() const {return m_multi_quants; } 
-    
+        bool UsingMultiQuants() const {return m_multi_quants; }
+
         //! Return the index of the parent subband
         int Parent() const {return m_parent;}
-    
+
         //! Return the indices of any child subbands
         const std::vector<int>& Children() const {return m_children;}
 
-        //! Return the index of a specific child band    
+        //! Return the index of a specific child band
         int Child(const int n) const {return m_children[n];}
 
         //! Return the code blocks
-        TwoDArray<CodeBlock>& GetCodeBlocks(){ return m_code_block_array; } 
+        TwoDArray<CodeBlock>& GetCodeBlocks(){ return m_code_block_array; }
 
         //! Return the code blocks
-        const TwoDArray<CodeBlock>& GetCodeBlocks() const { return m_code_block_array; } 
+        const TwoDArray<CodeBlock>& GetCodeBlocks() const { return m_code_block_array; }
 
         //! Returns true if subband is skipped, false if not
         bool Skipped() const { return m_skipped; }
-    
+
         //! Set the perceptual weight
         void SetWt( const float w );
-    
+
         //! Set the parent index
         void SetParent( const int p ){ m_parent=p; }
-    
+
         //! Set the subband depth
         void SetDepth( const int d ){ m_depth=d;}
-    
+
         //! Set the index of the maximum bit of the largest coefficient
         void SetMax( const int m ){ m_max_bit=m; };
-    
+
         //! Set the indices of the children of the subband
         void SetChildren( const std::vector<int>& clist ){ m_children = clist; }
-    
+
         //! Add a child to the list of child subbands
         void AddChild( const int c ){ m_children.push_back(c); }
 
@@ -257,7 +251,7 @@ namespace dirac
 
     private:
         // subband bounds
-        int m_xp , m_yp , m_xl , m_yl; 
+        int m_xp , m_yp , m_xl , m_yl;
 
         // perceptual weight for quantisation
         double m_wt;
@@ -275,7 +269,7 @@ namespace dirac
         std::vector<int> m_children;
 
         // position of the MSB of the largest absolute value
-        int m_max_bit;              
+        int m_max_bit;
 
         // The code blocks
         TwoDArray<CodeBlock> m_code_block_array;
@@ -300,28 +294,28 @@ namespace dirac
         //Default (shallow) copy constructor and operator= used
         //! Initialise the list
         void Init(const int depth,const int xlen,const int ylen);
-    
-        //! Return the length of the subband list    
+
+        //! Return the length of the subband list
         int Length() const {return bands.size();}
-    
+
         //! Return the subband at position n (1<=n<=length)
         Subband& operator()(const int n){return bands[n-1];}
-    
-        //! Return the subband at position n (1<=n<=length)    
-        const Subband& operator()(const int n) const {return bands[n-1];}    
-    
+
+        //! Return the subband at position n (1<=n<=length)
+        const Subband& operator()(const int n) const {return bands[n-1];}
+
         //! Add a band to the list
         void AddBand(const Subband& b){bands.push_back(b);}
-        
-        //! Remove all the bands from the list    
+
+        //! Remove all the bands from the list
         void Clear(){bands.clear();}
-    
+
     private:
 
         //! Given x and y spatial frequencies in cycles per degree, returns a weighting value
         float PerceptualWeight( const float xf , const float yf , const CompSort cs);
 
-    private:    
+    private:
         std::vector<Subband> bands;
     };
 
@@ -334,8 +328,8 @@ namespace dirac
     {
     public:
         //! Constructor
-        WaveletTransform(int d = 4, WltFilter f = DAUB97);
-        
+        WaveletTransform(int d = 4, WltFilter f = DAUB9_7);
+
         //! Destructor
         virtual ~WaveletTransform();
 
@@ -344,34 +338,35 @@ namespace dirac
             Transforms the data to and from the wavelet domain.
             \param    d    the direction of the transform
             \param    pic_data    the data to be transformed
+            \param    coeff_data  array that holds the transform coefficient data
         */
-        void Transform(const Direction d, PicArray& pic_data);
-    
+        void Transform(const Direction d, PicArray& pic_data, CoeffArray& coeff_data);
+
         //! Returns the set of subbands
         SubbandList& BandList(){return m_band_list;}
-    
+
         //! Returns the set of subbands
         const SubbandList& BandList() const {return m_band_list;}
-    
+
         //! Sets the subband weights
         /*!
             Sets perceptual weights for the subbands. Takes into account both perceptual factors
-            (weight noise less at higher spatial frequencies) and the scaling needed for the 
-            wavelet transform. 
+            (weight noise less at higher spatial frequencies) and the scaling needed for the
+            wavelet transform.
 
-            \param    cpd    perctual factor - the number of cycles per degree
-            \param    fsort    the frame sort (I, L1 or L2)
-            \param    cformat    the chroma format
-            \param    csort    the component type (Y, U or V)  
+            \param    cpd          perctual factor - the number of cycles per degree
+            \param    fsort        the frame sort (I, L1 or L2)
+            \param    cformat      the chroma format
+            \param    csort        the component type (Y, U or V)
+            \param    field_coding True if input is coded as fields
         */
-        void SetBandWeights (const float cpd, 
+        void SetBandWeights (const float cpd,
                              const FrameSort& fsort,
                              const ChromaFormat& cformat,
-                             const CompSort csort);
+                             const CompSort csort,
+                             const bool field_coding);
+
 
-        //! Return the expected DC band value assuming a mid-grey picture (mean val=512)
-        ValueType GetMeanDCVal() const;
- 
     private:
         // Classes used within wavelet transform
 
@@ -386,10 +381,10 @@ namespace dirac
             virtual ~VHFilter(){}
 
             //! Split a subband into 4
-            virtual void Split(const int xp, const int yp, const int xl, const int yl, PicArray&pic_data)=0; 
+            virtual void Split(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data)=0;
 
             //! Create a single band from 4 quadrant bands
-            virtual void Synth(const int xp, const int yp, const int xl, const int yl, PicArray& pic_data)=0;
+            virtual void Synth(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data)=0;
 
             //! Return a correction factor to compensate for non-unity gain of low-pass filter
             virtual double GetLowFactor() const=0;
@@ -397,27 +392,36 @@ namespace dirac
             //! Return a correction factor to compensate for non-unity gain of high-pass filter
             virtual double GetHighFactor() const =0;
 
+            //! Return the value of the additional bitshift
+            virtual int GetShift() const =0;
+
         protected:
-            
+
             //! Interleave data from separate subbands into even and odd positions for in-place calculation - called by Synth
-            inline void Interleave( const int xp, const int yp, const int xl, const int yl, PicArray&pic_data );
-    
+            inline void Interleave( const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data );
+
 
             //! De-interleave data even and odd positions into separate subbands - called by Split
-            inline void DeInterleave( const int xp, const int yp, const int xl, const int yl, PicArray&pic_data );
+            inline void DeInterleave( const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data );
+
+            //! Shift all vals in Row by 'shift' bits to the left to increase accuracy by 'shift' bits. Used in Analysis stage of filter
+            void ShiftRowLeft(CoeffType *row, int length, int shift);
+
+        //! Shift all vals in Row by 'shift' bits to the right to counter the shift in the Analysis stage. This function is used in the Synthesis stage
+            void ShiftRowRight(CoeffType *row, int length, int shift);
         };
 
         //! Class to do Daubechies (9,7) filtering operations
-        class VHFilterDaub9_7 : public VHFilter
+        class VHFilterDAUB9_7 : public VHFilter
         {
 
         public:
 
             //! Split a subband into 4
-            void Split(const int xp, const int yp, const int xl, const int yl, PicArray&pic_data); 
+            void Split(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
 
             //! Create a single band from 4 quadrant bands
-            void Synth(const int xp, const int yp, const int xl, const int yl, PicArray& pic_data);
+            void Synth(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
 
             //! Return a correction factor to compensate for non-unity gain of low-pass filter
             double GetLowFactor() const { return 1.149604398;}
@@ -425,43 +429,51 @@ namespace dirac
             //! Return a correction factor to compensate for non-unity gain of high-pass filter
             double GetHighFactor() const { return 0.869864452;}
 
+            //! Return the value of the additional bitshift
+            int GetShift() const {return 1;}
+
+
         };
 
         //! Class to do (5,3) wavelet filtering operations
-        class VHFilter5_3 : public VHFilter
+        class VHFilterLEGALL5_3 : public VHFilter
         {
 
         public:
 
             //! Split a subband into 4
-            void Split(const int xp, const int yp, const int xl, const int yl, PicArray&pic_data); 
+            void Split(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
 
             //! Create a single band from 4 quadrant bands
-            void Synth(const int xp, const int yp, const int xl, const int yl, PicArray& pic_data);
+            void Synth(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
 
             //! Return a correction factor to compensate for non-unity power gain of low-pass filter
-            double GetLowFactor() const { return 1.179535649;}    
+            double GetLowFactor() const { return 1.179535649;}
 
             //! Return a correction factor to compensate for non-unity power gain of high-pass filter
             double GetHighFactor() const { return 0.81649658;}
 
+            //! Return the value of the additional bitshift
+            int GetShift() const {return 1;}
+
+
 #ifdef HAVE_MMX
-            inline void HorizSynth (int xp, int xl, int ystart, int yend, PicArray &pic_data);
+            inline void HorizSynth (int xp, int xl, int ystart, int yend, CoeffArray &coeff_data);
 #endif
 
         };
 
-        //! Class to do an approximation to Daubechies (9,7) but with just two lifting steps
-        class VHFilterApprox9_7 : public VHFilter
+        //! A short filter that's actually close to Daubechies (9,7) but with just two lifting steps
+        class VHFilterDD9_7 : public VHFilter
         {
 
         public:
 
             //! Split a subband into 4
-            void Split(const int xp, const int yp, const int xl, const int yl, PicArray&pic_data); 
+            void Split(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
 
             //! Create a single band from 4 quadrant bands
-            void Synth(const int xp, const int yp, const int xl, const int yl, PicArray& pic_data);
+            void Synth(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
 
             //! Return a correction factor to compensate for non-unity power gain of low-pass filter
             double GetLowFactor() const { return 1.218660804;}
@@ -469,28 +481,108 @@ namespace dirac
             //! Return a correction factor to compensate for non-unity power gain of high-pass filter
             double GetHighFactor() const { return 0.780720058;}
 
+            //! Return the value of the additional bitshift
+            int GetShift() const {return 1;}
+
         };
 
 
-        //! An extension of Approx9_7, with a better low-pass filter but more computation
-        class VHFilter13_5 : public VHFilter
+        //! An extension of DD9_7, with a better low-pass filter but more computation
+        class VHFilterDD13_7 : public VHFilter
         {
 
         public:
 
             //! Split a subband into 4
-            void Split(const int xp, const int yp, const int xl, const int yl, PicArray&pic_data); 
+            void Split(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
 
             //! Create a single band from 4 quadrant bands
-            void Synth(const int xp, const int yp, const int xl, const int yl, PicArray& pic_data);
+            void Synth(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
 
             //! Return a correction factor to compensate for non-unity power gain of low-pass filter
-            double GetLowFactor() const { return 1.28087;}
+            double GetLowFactor() const { return 1.235705971;}
 
             //! Return a correction factor to compensate for non-unity power gain of high-pass filter
-            double GetHighFactor() const { return 0.809254;}
+            double GetHighFactor() const { return 0.780719354;}
+
+            //! Return the value of the additional bitshift
+            int GetShift() const {return 1;}
+
         };
 
+        //! Class to do Haar wavelet filtering operations
+        class VHFilterHAAR0 : public VHFilter
+        {
+
+        public:
+
+            //! Split a subband into 4
+            void Split(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
+
+            //! Create a single band from 4 quadrant bands
+            void Synth(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
+
+            //! Return a correction factor to compensate for non-unity power gain of low-pass filter
+            double GetLowFactor() const { return 1.414213562;}
+
+            //! Return a correction factor to compensate for non-unity power gain of high-pass filter
+            double GetHighFactor() const { return 0.707106781;}
+
+            //! Return the value of the additional bitshift
+            int GetShift() const {return 0;}
+
+
+        };
+
+        //! Class to do Haar wavelet filtering operations with a single shift per level
+        class VHFilterHAAR1 : public VHFilter
+        {
+
+        public:
+
+            //! Split a subband into 4
+            void Split(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
+
+            //! Create a single band from 4 quadrant bands
+            void Synth(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
+
+            //! Return a correction factor to compensate for non-unity power gain of low-pass filter
+            double GetLowFactor() const { return 1.414213562;}
+
+            //! Return a correction factor to compensate for non-unity power gain of high-pass filter
+            double GetHighFactor() const { return 0.707106781;}
+
+            //! Return the value of the additional bitshift
+            int GetShift() const {return 1;}
+
+
+        };
+
+
+       //! Class to do Haar wavelet filtering operations with a double shift per level
+        class VHFilterHAAR2 : public VHFilter
+        {
+
+        public:
+
+            //! Split a subband into 4
+            void Split(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
+
+            //! Create a single band from 4 quadrant bands
+            void Synth(const int xp, const int yp, const int xl, const int yl, CoeffArray& coeff_data);
+
+            //! Return a correction factor to compensate for non-unity power gain of low-pass filter
+            double GetLowFactor() const { return 1.414213562;}
+
+            //! Return a correction factor to compensate for non-unity power gain of high-pass filter
+            double GetHighFactor() const { return 0.707106781;}
+
+            //! Return the value of the additional bitshift
+            int GetShift() const {return 2;}
+
+        };
+
+
 
         // Lifting steps used in the filters
 
@@ -513,9 +605,9 @@ namespace dirac
                 \param   val1   the first value being used for prediction
                 \param   val2   the second value being used for prediction
             */
-            inline void Filter(ValueType& in_val, const ValueType& val1, const ValueType& val2) const
+            inline void Filter(CoeffType& in_val, const CoeffType& val1, const CoeffType& val2) const
             {
-                in_val -= (( val1 + val2 ) >>shift );
+                in_val -= (( val1 + val2 + (1<<(shift-1)) ) >>shift );
             }
 
         };
@@ -536,12 +628,12 @@ namespace dirac
                 \param   val1   the first value being used for updating
                 \param   val2   the second value being used for updating
             */
-            inline void Filter(ValueType& in_val, const ValueType& val1, const ValueType& val2) const
+            inline void Filter(CoeffType& in_val, const CoeffType& val1, const CoeffType& val2) const
             {
-                in_val += ( ( val1 + val2 ) >>shift );
+                in_val += ( ( val1 + val2 + (1<<(shift-1)) ) >>shift );
             }
 
-        };  
+        };
 
         //! Class to do symmetric four-tap prediction lifting step
         template <int shift , int tap1, int tap2>
@@ -555,29 +647,29 @@ namespace dirac
             // Assume default copy constructor, assignment= and destructor //
 
             //! Do the filtering
-            inline void Filter(ValueType& in_val, const ValueType& val1, const ValueType& val2 ,
-                                                  const ValueType& val3, const ValueType& val4 ) const
+            inline void Filter(CoeffType& in_val, const CoeffType& val1, const CoeffType& val2 ,
+                                                  const CoeffType& val3, const CoeffType& val4 ) const
             {
-                in_val -= ( tap1*( val1 + val2 ) + tap2*( val3 + val4 ) )>>shift;
+                in_val -= ( tap1*( val1 + val2 ) + tap2*( val3 + val4 ) + (1<<(shift-1)))>>shift;
             }
-        }; 
+        };
 
         //! Class to do symmetric four-tap update lifting step
         template <int shift , int tap1 , int tap2>
         class UpdateStepFourTap
         {
- 
+
         public:
             //! Constructor
             UpdateStepFourTap(){}
 
             //! Do the filtering
-            inline void Filter(ValueType& in_val, const ValueType& val1, const ValueType& val2 ,
-                                                  const ValueType& val3, const ValueType& val4 ) const
+            inline void Filter(CoeffType& in_val, const CoeffType& val1, const CoeffType& val2 ,
+                                                  const CoeffType& val3, const CoeffType& val4 ) const
             {
-                in_val += ( tap1*( val1 + val2 ) + tap2*( val3 + val4 ) )>>shift;
+                in_val += ( tap1*( val1 + val2 ) + tap2*( val3 + val4 ) + (1<<(shift-1)) )>>shift;
             }
-        };  
+        };
 
         //! Class to do two-tap prediction lifting step for Daubechies (9,7)
         template <int gain> class PredictStep97
@@ -596,16 +688,16 @@ namespace dirac
                 \param   val1   the first value being used for prediction
                 \param   val2   the second value being used for prediction
             */
-            inline void Filter(ValueType& in_val, const ValueType& val1, const ValueType& val2) const
+            inline void Filter(CoeffType& in_val, const CoeffType& val1, const CoeffType& val2) const
             {
-                in_val -= static_cast< ValueType >( (gain * static_cast< int >( val1 + val2 )) >>12 );
+                in_val -= static_cast< CoeffType >( (gain * static_cast< int >( val1 + val2 )) >>12 );
             }
-        }; 
+        };
 
         //! Class to do two-tap update lifting step for Daubechies (9,7)
         template <int gain> class UpdateStep97
         {
- 
+
         public:
             //! Constructor
             UpdateStep97(){}
@@ -617,23 +709,23 @@ namespace dirac
                 \param   val1   the first value being used for updating
                 \param   val2   the second value being used for updating
             */
-            inline void Filter(ValueType& in_val, const ValueType& val1, const ValueType& val2) const
+            inline void Filter(CoeffType& in_val, const CoeffType& val1, const CoeffType& val2) const
             {
-                in_val += static_cast< ValueType >( (gain * static_cast< int >( val1 + val2 )) >>12 );
+                in_val += static_cast< CoeffType >( (gain * static_cast< int >( val1 + val2 )) >>12 );
             }
-        };  
+        };
 
     private:
 
-        // Private variables    
+        // Private variables
 
         SubbandList m_band_list;
 
         //! Depth of the transform
         int m_depth;
-    
+
         //! The filter set to be used
-        WltFilter m_filt_sort; 
+        WltFilter m_filt_sort;
 
         //! A class to do the filtering required
         VHFilter* m_vhfilter;
@@ -642,13 +734,13 @@ namespace dirac
         // Private functions
         //!    Private, bodyless copy constructor: class should not be copied
         WaveletTransform(const WaveletTransform& cpy);
-    
+
         //! Private, bodyless copy operator=: class should not be assigned
         WaveletTransform& operator=(const WaveletTransform& rhs);
-    
+
         //! Given x and y spatial frequencies in cycles per degree, returns a weighting value
         float PerceptualWeight(float xf,float yf,CompSort cs);
-   };  
+   };
 
 }// end namespace dirac
 
