@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: dirac_cppparser.h,v 1.3 2004/11/22 14:05:03 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: dirac_cppparser.h,v 1.5 2007/09/03 11:31:42 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -21,6 +21,7 @@
 * All Rights Reserved.
 *
 * Contributor(s): Anuradha Suraparaju (Original Author)
+*                 Andrew Kennedy
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -44,6 +45,7 @@
 #include <streambuf>
 #include <libdirac_decoder/decoder_types.h> //for DecoderState
 #include <libdirac_common/common.h> 
+#include <libdirac_byteio/dirac_byte_stream.h>  
 
 namespace dirac
 {
@@ -127,7 +129,7 @@ namespace dirac
         //! Destructor
         ~DiracParser();
         
-        //! Copy data into the internal stream buffer
+        //! Adds bytes to encoder
         /*! SetBuffer takes
             \param start   Start of input buffer
             \param end     End of input buffer
@@ -147,17 +149,23 @@ namespace dirac
         */
         DecoderState Parse();
 
-        //! Return the sequence parameters of the current sequence
-        const SeqParams& GetSeqParams() const;
+        //! Return the parse parameters of the current sequence
+        const ParseParams& GetParseParams() const;
+
+        //! Return the source parameters of the current sequence
+        const SourceParams& GetSourceParams() const;
 
         //! Return the frame parameters of the next frame to be decoded
         const FrameParams& GetNextFrameParams() const;
 
         //! Return the decoded frame
         const Frame& GetNextFrame() const;
+        
+        //! Return the coding parameters of the current sequence
+        const DecoderParams& GetDecoderParams() const;
 
         //! Return the last frame in the sequence
-        const Frame& GetLastFrame() const;
+      //  const Frame& GetLastFrame() const;
 
         //! Set the skip flag
         /*! Set the skip flag to the value specified in skip. If skip is true,
@@ -168,12 +176,6 @@ namespace dirac
 
     private:
 
-        //! Determine if enough data is available in internal buffer to process
-        DecoderState SeekChunk();
-
-        //! Initialise the parser's internal state variables
-        void InitStateVars();
-
     private:
 
         //! private body-less copy constructor
@@ -188,22 +190,14 @@ namespace dirac
         int m_show_fnum;
         //! Sequence decompressor object
         SequenceDecompressor *m_decomp;
-        //! Input stream object. Initialised using the external input buffer InputStreamBuffer
-        std::istream *m_istr;
-        //! Internal Stream Buffer
-        InputStreamBuffer m_sbuf;
         //! skip next frame flag
         bool m_skip;
         //! skip frame type
         FrameSort m_skip_type;
         //! verbose flag
         bool m_verbose;
-        //! start of chunk flag
-        bool m_found_start;
-        //! end of chunk flag
-        bool m_found_end;
-        //! used to detect start and end of chunk
-        unsigned m_shift;
+        //! Byte Stream Buffer
+        DiracByteStream m_dirac_byte_stream;
     };
 
 } // namespace dirac
