@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: frame_decompress.h,v 1.11 2005/05/19 10:56:57 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: frame_decompress.h,v 1.17 2007/09/03 11:31:42 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -23,6 +23,7 @@
 * Contributor(s): Thomas Davies (Original Author),
 *                 Scott R Ladd,
 *                 Anuradha Suraparaju
+*                 Andrew Kennedy
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -44,9 +45,13 @@
 
 #include <libdirac_common/frame_buffer.h>
 #include <libdirac_common/common.h>
+#include <libdirac_byteio/frame_byteio.h>
+#include <libdirac_byteio/transform_byteio.h>
 
 namespace dirac
 {
+    class MvData;
+    
     //! Compress a single image frame
     /*!
         This class decompresses a single frame at a time, using parameters
@@ -78,16 +83,11 @@ namespace dirac
             of a frame buffer.
             Returns true if able to decode successfully, false otherwise
 
+            \param parseunit_byteio Frame info in Dirac-stream format
             \param my_buffer   picture buffer into which the frame is placed
         */
-        bool Decompress(FrameBuffer& my_buffer);
-
-        //! Reads the header data
-        /*!
-            Reads the header data associated with decompressing the frame
-            \param my_buffer picture buffer from which frame dimensions are obtained
-        */
-        bool ReadFrameHeader(const FrameBuffer& my_buffer);
+        bool Decompress(ParseUnitByteIO& parseunit_byteio,
+                        FrameBuffer& my_buffer);
 
         //! Returns the frame parameters of the current frame being decoded
         const FrameParams& GetFrameParams() const{ return m_fparams; }
@@ -107,11 +107,22 @@ namespace dirac
         */
         FrameDecompressor& operator=(const FrameDecompressor& rhs);
 
-        //! Decodes component data    
-        void CompDecompress(FrameBuffer& my_buffer,int fnum, CompSort cs);
+        //! Removes all the reference frames in the retired list
+        void CleanReferenceFrames( FrameBuffer& my_buffer );
 
-        //! Reads the header data associated with decompressing the frame
-        bool ReadFrameHeader(FrameParams& fparams);
+        //! Decodes component data    
+        void CompDecompress(TransformByteIO *p_transform_byteio,
+                            FrameBuffer& my_buffer,int fnum, CompSort cs);
+
+        //! Decodes the motion data
+        void DecompressMVData( std::auto_ptr<MvData>& mv_data, FrameByteIO& frame_byteio );
+         
+
+        //! Set the number of superblocks and blocks
+        void SetMVBlocks();
+
+        //! Add a frame to the frame buffer
+        void PushFrame(FrameBuffer &my_buffer);
 
         //Member variables    
 
@@ -135,10 +146,6 @@ namespace dirac
 
         //! Current Frame Parameters
         FrameParams m_fparams;
-
-        //! Read header successfully
-        bool m_read_header;
-
     };
 
 } // namespace dirac
