@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: seq_decompress.h,v 1.7 2004/11/29 12:55:05 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: seq_decompress.h,v 1.10 2007/09/03 11:31:43 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -23,6 +23,7 @@
 * Contributor(s): Thomas Davies (Original Author),
 *                 Scott R Ladd,
 *                 Anuradha Suraparaju
+*                 Andrew Kennedy
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -48,6 +49,7 @@
 ///////////////////////////////////////////
 
 #include "libdirac_common/common.h"
+#include "libdirac_byteio/parseunit_byteio.h"
 #include <iostream>
 
 namespace dirac
@@ -67,10 +69,10 @@ namespace dirac
         /*!
             Initializes the decompressor with an input stream and level of
             output detail.
-            \param  ip          input data stream containing a sequence of compressed images
+            \param  parseunit   First access-unit of new sequence
             \param  verbosity   when true, increases the amount of information displayed during decompression
          */
-        SequenceDecompressor(std::istream * ip, bool verbosity);
+        SequenceDecompressor(ParseUnitByteIO& parseunit, bool verbosity);
 
         //! Destructor
         /*!
@@ -78,6 +80,13 @@ namespace dirac
         */
         ~SequenceDecompressor();
 
+        //! Marks beginning of a new AccessUnit
+        /*!
+            \param parseunit_byteio AccessUnit info in Dirac-stream format
+        */
+        void NewAccessUnit(ParseUnitByteIO& parseunit_byteio);
+
+
         //! Decompress the next frame in sequence
         /*!
             This function decodes the next frame in coding order and returns
@@ -90,13 +99,12 @@ namespace dirac
             with the decoded frames as they come out -- write them to screen
             or to file, as required.
 
+            \param p_parseunit_byteio Frame information in Dirac-stream format
             \param  skip skip decoding next frame
             \return      reference to the next locally decoded frame available for display
         */
-        Frame& DecompressNextFrame(bool skip = false);
-
-        //! Reads the header data associated with decompressing the frame
-        bool ReadNextFrameHeader();
+        Frame& DecompressNextFrame(ParseUnitByteIO* p_parseunit_byteio,
+                                   bool skip = false);
 
         //! Get the next frame available for display
         Frame& GetNextFrame();
@@ -109,16 +117,32 @@ namespace dirac
             decompressed.
             \return     true if last frame has been compressed; false if not
         */
-        bool Finished() { return m_all_done; }
-
-        //! Interrogates for decompression parameters.
+        bool Finished(); 
+        //! Interrogates for parse parameters.
         /*!
-            Returns the parameters used for this decompression run.
+            Returns the parse parameters used for this decompression run.
 
-            \return decompression parameters originally provide din the constructor.
+            \return parse parameters.
          */
-        SeqParams & GetSeqParams() { return m_sparams; }
+        ParseParams & GetParseParams() { return m_parse_params; }
 
+
+        //! Interrogates for source parameters.
+        /*!
+            Returns the source parameters used for this decompression run.
+
+            \return source parameters.
+         */
+        SourceParams & GetSourceParams() { return m_srcparams; }
+
+
+        //! Interrogates for coding parameters.
+        /*!
+            Returns the decoder parameters used for this decompression run.
+
+            \return decoder parameters.
+         */
+        DecoderParams & GetDecoderParams() { return m_decparams; }
     private:
         //! Copy constructor is private and body-less
         /*!
@@ -136,14 +160,6 @@ namespace dirac
         */
         SequenceDecompressor& operator=(const SequenceDecompressor& rhs);
 
-        //! Read a sequence header from bitstream
-        /*!
-            Reads the sequence data from the bitstream. This contains all the
-            block information. Temporal prediction information is contained in
-            the frame headers so that a simple GOP need not be used, or if so,
-            can be reset on the fly.
-         */
-        void ReadStreamHeader();    
 
         //Member variables
 
@@ -151,12 +167,12 @@ namespace dirac
         bool m_all_done;
         //! Parameters for the decompression, as provided in constructor
         DecoderParams m_decparams;
-        //! The sequence parameters obtained from the stream header
-        SeqParams m_sparams;
+        //! The parse parameters obtained from the stream header
+        ParseParams m_parse_params;
+        //! The source parameters obtained from the stream header
+        SourceParams m_srcparams;
         //! A picture buffer used for local storage of frames whilst pending re-ordering or being used for reference.
-        FrameBuffer* m_fbuffer;
-        //! Input file pointer, pointing at the bitstream
-        std::istream* m_infile;            
+        FrameBuffer* m_fbuffer;   
         //! Number of the frame in coded order which is to be decoded
         int m_current_code_fnum;        
         //! A delay so that we don't display what we haven't decoded
@@ -167,6 +183,8 @@ namespace dirac
         int m_show_fnum;
         //! Frame decompressor object
         FrameDecompressor *m_fdecoder;
+        //! Highest frame-num processed - for tracking end-of-sequence
+        int m_highest_fnum;
     };
 
 } // namespace dirac
