@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: comp_compress.h,v 1.8 2005/02/02 14:47:18 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: comp_compress.h,v 1.16 2008/01/22 07:38:37 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -21,6 +21,7 @@
 * All Rights Reserved.
 *
 * Contributor(s): Thomas Davies (Original Author), Scott R Ladd
+*                 Andrew Kennedy
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -42,9 +43,12 @@
 #include <libdirac_common/arrays.h>
 #include <libdirac_common/wavelet_utils.h>
 #include <libdirac_common/common.h>
+#include <libdirac_byteio/component_byteio.h>
 
 namespace dirac
 {
+    class MEData;
+          
     //! Compress a frame component
     /*!
         This class compresses one of the three components (Y, U, or V) of a
@@ -66,9 +70,14 @@ namespace dirac
         //! Compress a frame component
         /*!
             Compress a PicArray containing a frame component (Y, U, or V).
-            \param  pic_data    the component data to be compressed
+            \param  coeff_data      the component data to be compressed
+            \param  bands           Subbands list
+            \param  estimated_bits  the list of estimated number of bits in each subband
+            \return Frame-component in Dirac-bytestream format
         */
-        void Compress(PicArray & pic_data);
+        ComponentByteIO* Compress( CoeffArray& coeff_data ,
+                                   SubbandList& bands,
+                                   const OneDArray<unsigned int>& estimated_bits);
 
     private:
         //! Copy constructor is private and body-less. This class should not be copied.
@@ -76,43 +85,26 @@ namespace dirac
 
         //! Assignment = is private and body-less. This class should not be assigned.
         CompCompressor& operator=(const CompCompressor& rhs);
+        
+        //! Set a subband to a constant value
+        void SetToVal(CoeffArray& coeff_data,const Subband& node,ValueType val);
 
-        void SelectQuantisers( PicArray& pic_data , 
-                               SubbandList& bands ,
-                               OneDArray<unsigned int>& est_counts,
-                               const bool using_multi_quants );
 
-        int SelectMultiQuants( PicArray& pic_data , 
-                               SubbandList& bands , 
-                               const int band_num );
-
-        //! Write the header data of the subband to the output
-        /* 
-            Write the header data of the subband to the output. This includes
-            whether the subband is skipped (if yes, there's no more data),
-            the subband quantiser index, whether multiple quantisers are used.
-            \param output_mgr the output buffer
-            \param node the subband metadata
-            \param num_band_bytes the length of the subband data in bytes
-
-        */
-        void WriteBandHeader( BasicOutputManager& output_mgr , const Subband& node ,
-                              const int num_band_bytes );
-
-        void SetupCodeBlocks( SubbandList& bands );
-
-        void SetToVal(PicArray& pic_data,const Subband& node,ValueType val);
-
-        void AddSubAverage(PicArray& pic_data,int xl,int yl,AddOrSub dirn);
+    private:
 
         // member variables
         EncoderParams& m_encparams;
+
         const FrameParams& m_fparams;
+
         const FrameSort& m_fsort;    
+
         const ChromaFormat& m_cformat;
+
         CompSort m_csort;
 
         float m_lambda;
+        
     };
 
 } // namespace dirac
