@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: frame_compress.h,v 1.10 2005/09/27 09:16:20 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: frame_compress.h,v 1.16 2007/12/12 14:01:51 tjdwave Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -23,6 +23,7 @@
 * Contributor(s): Thomas Davies (Original Author),
 *                 Scott R Ladd,
 *                 Anuradha Suraparaju
+*                 Andrew Kennedy
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -44,6 +45,8 @@
 #include <libdirac_common/frame_buffer.h>
 #include <libdirac_common/common.h>
 #include <libdirac_common/motion.h>
+#include <libdirac_byteio/frame_byteio.h>
+
 namespace dirac
 {
 
@@ -69,14 +72,25 @@ namespace dirac
 
         //! Destructor
         ~FrameCompressor( );
+
+        //! Performs motion estimation for a frame and writes the data locally
+        /*! Performs motion estimation for a frame and writes the data locally
+            \param my_fbuffer picture buffer of uncoded originals
+            \param fnum    frame number to compress
+            \return true   if a cut is detected.
+        */                        
+        bool MotionEstimate( const FrameBuffer& my_fbuffer, 
+                                        int fnum); 
+
         //! Compress a specific frame within a group of pictures (GOP)
         /*!
             Compresses a specified frame within a group of pictures. 
-            \param fbuffer picture buffer in which the frame resides
-            \param orig_buffer the corresponding picture buffer of uncoded originals
-            \param fnum      frame number to compress
+            \param my_fbuffer  picture buffer in which the reference frames resides
+            \param fnum        frame number to compress
+            \return Compressed frame in Dirac bytestream format
         */
-        void Compress( FrameBuffer& fbuffer , const FrameBuffer& orig_buffer , int fnum );
+        FrameByteIO* Compress(  FrameBuffer& my_fbuffer , 
+                                int fnum );
 
         //! Returns true if the frame has been skipped rather than coded normally
         bool IsSkipped(){ return m_skipped; }
@@ -102,11 +116,35 @@ namespace dirac
         */
         FrameCompressor& operator=(const FrameCompressor& rhs);
 
-        //! Write the frame compression header
-        void WriteFrameHeader(const FrameParams& fparams);
-
         //! Analyses the ME data and returns true if a cut is detected, false otherwise
         void AnalyseMEData( const MEData& );
+        
+        //! Compresses the motion vector data
+        void CompressMVData(MvDataByteIO* mv_data);
+        
+        //! Returns the value lambda according to frame and component type
+        float GetCompLambda( const FrameParams& fparams,
+                             const CompSort csort );
+
+        void SelectQuantisers( CoeffArray& coeff_data , 
+                               SubbandList& bands ,
+                               const float lambda,
+                               OneDArray<unsigned int>& est_counts,
+                               const CodeBlockMode cb_mode,
+                               const FrameSort fsort,
+                               const CompSort csort );
+
+        int SelectMultiQuants( CoeffArray& coeff_data , 
+                               SubbandList& bands , 
+                               const int band_num,
+                               const float lambda,
+                               const FrameSort fsort, 
+                               const CompSort csort );
+
+        void SetupCodeBlocks( SubbandList& bands );
+
+
+        void AddSubAverage(CoeffArray& coeff_data,int xl,int yl,AddOrSub dirn);
 
     private:
 
