@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: quality_monitor.h,v 1.11 2005/08/10 08:02:36 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: quality_monitor.h,v 1.14 2007/09/03 11:31:43 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -53,8 +53,7 @@ namespace dirac
         /*
            Constructor sets up initial Lagrangian values.
         */
-        QualityMonitor(EncoderParams& ep,
-                       const SeqParams& sparams );
+        QualityMonitor(EncoderParams& ep);
 
 
         //! Destructor
@@ -86,11 +85,10 @@ namespace dirac
 
         //! Calculate the quality of coded wrt original picture
         double QualityVal( const PicArray& coded_data , 
-                                           const PicArray& orig_data ,
-                                           double cpd , 
-                                           const FrameSort fsort );
-
-        ValueType Filter( const PicArray& data , const int xpos , const int ypos ) const;
+                           const PicArray& orig_data,
+                           const int xlen,
+                           const int ylen,
+                           const CompSort cs);
 
         //member variables//
         ////////////////////
@@ -98,17 +96,26 @@ namespace dirac
         //! A reference to the encoder parameters
         EncoderParams& m_encparams;
 
-        //! The chroma format
-        const ChromaFormat m_cformat;
+        //! The overall average Y quality
+        long double m_totalquality_averageY;
 
-        //! The true picture width, minus padding
-        const int m_true_xl;
+        //! The overall average U quality
+        long double m_totalquality_averageU;
 
-        //! The true picture height, minus padding
-        const int m_true_yl;
+        //! The overall average V quality
+        long double m_totalquality_averageV;
 
-        //! The average quality for the frame types
-        OneDArray<long double> m_quality_average;
+        //! The total number of frames coded
+        int m_allframe_total;        
+
+        //! The average Y quality for the frame types
+        OneDArray<long double> m_quality_averageY;
+
+        //! The average U quality for the frame types
+        OneDArray<long double> m_quality_averageU;
+
+        //! The average V quality for the frame types
+        OneDArray<long double> m_quality_averageV;
 
         //! The number of frames of each type  
         OneDArray<int> m_frame_total;
