@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: downconvert.cpp,v 1.9 2005/11/08 11:37:44 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: downconvert.cpp,v 1.10 2007/03/19 16:19:00 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -87,7 +87,7 @@ void DownConverter::DoDownConvert(const PicArray& old_data, PicArray& new_data)
         }// x
         //Speaking of which - the row loop.
 
-        RowLoop(colpos,old_data,new_data);
+        RowLoop(colpos,new_data);
     }// y 
 
     // This loop is like the last one but it deals with the center
@@ -108,7 +108,7 @@ void DownConverter::DoDownConvert(const PicArray& old_data, PicArray& new_data)
             m_row_buffer[x] = sum >> StageI_Shift;
         }// x
 
-        RowLoop( colpos , old_data , new_data );
+        RowLoop( colpos , new_data );
     }// y
 
     // Another similar loop! - this time we are dealing with
@@ -133,7 +133,7 @@ void DownConverter::DoDownConvert(const PicArray& old_data, PicArray& new_data)
 
         }// x
 
-        RowLoop( colpos , old_data , new_data );
+        RowLoop( colpos , new_data );
 
     }//  y
 
@@ -146,7 +146,7 @@ void DownConverter::DoDownConvert(const PicArray& old_data, PicArray& new_data)
 
 // The loop over the columns is the same every time so lends itself to isolation
 // as an individual function.
-void DownConverter::RowLoop( const int colpos , const PicArray& old_data , PicArray& new_data)
+void DownConverter::RowLoop( const int colpos , PicArray& new_data)
 {
 
      //Calculation variables
