@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: downconvert.h,v 1.10 2004/11/29 12:55:06 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: downconvert.h,v 1.11 2007/03/19 16:19:00 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -74,7 +74,7 @@ namespace dirac
         DownConverter& operator=(const DownConverter& rhs);//private, body-less: class should not be assigned
 
         //Applies the filter to a single column
-        void RowLoop(const int colpos , const PicArray& old_data , PicArray& new_data );
+        void RowLoop(const int colpos , PicArray& new_data );
 
         ValueType* m_row_buffer;
 
