@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: me_mode_decn.cpp,v 1.17 2005/09/27 13:01:50 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: me_mode_decn.cpp,v 1.25 2007/08/13 10:29:23 tjdwave Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -20,7 +20,8 @@
 * Portions created by the Initial Developer are Copyright (C) 2004.
 * All Rights Reserved.
 *
-* Contributor(s): Thomas Davies (Original Author)
+* Contributor(s): Thomas Davies (Original Author),
+*                 Tim Borer
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -73,7 +74,7 @@ ModeDecider::ModeDecider( const EncoderParams& encp):
 
 ModeDecider::~ModeDecider()
 {
-    if (fsort != I_frame)
+    if (fsort.IsInter())
     {
         delete m_me_data_set[0];
         delete m_me_data_set[1];
@@ -93,7 +94,7 @@ void ModeDecider::DoModeDecn(const FrameBuffer& my_buffer, int frame_num, MEData
     ////////////////
 
     fsort = my_buffer.GetFrame(frame_num).GetFparams().FSort();
-    if (fsort != I_frame)
+    if (fsort.IsInter())
     {
         // Extract the references
         const vector<int>& refs = my_buffer.GetFrame(frame_num).GetFparams().Refs();
@@ -105,9 +106,9 @@ void ModeDecider::DoModeDecn(const FrameBuffer& my_buffer, int frame_num, MEData
 
         // Set up the hierarchy of motion vector data objects
         m_me_data_set[0] = new MEData( m_encparams.XNumMB() , m_encparams.YNumMB() , 
-                                       m_encparams.XNumBlocks()/4 , m_encparams.YNumBlocks()/4 );
+                                       m_encparams.XNumBlocks()/4 , m_encparams.YNumBlocks()/4, num_refs );
         m_me_data_set[1] = new MEData( m_encparams.XNumMB() , m_encparams.YNumMB() , 
-                                       m_encparams.XNumBlocks()/2 , m_encparams.YNumBlocks()/2 );
+                                       m_encparams.XNumBlocks()/2 , m_encparams.YNumBlocks()/2, num_refs );
 
         m_me_data_set[2] = &me_data;
 
@@ -124,11 +125,11 @@ void ModeDecider::DoModeDecn(const FrameBuffer& my_buffer, int frame_num, MEData
             m_ref2_updata = &(my_buffer.GetUpComponent( ref2 , Y_COMP));
             // Create an object for computing bi-directional prediction calculations            
 
-            if ( m_encparams.MVPrecision()==3 )
+            if ( m_encparams.MVPrecision()==MV_PRECISION_EIGHTH_PIXEL )
                 m_bicheckdiff = new BiBlockEighthPel( *m_ref1_updata ,
                                                       *m_ref2_updata ,
                                                       *m_pic_data );
-            else if ( m_encparams.MVPrecision()==2 )
+            else if ( m_encparams.MVPrecision()==MV_PRECISION_QUARTER_PIXEL )
                 m_bicheckdiff = new BiBlockQuarterPel( *m_ref1_updata ,
                                                        *m_ref2_updata ,
                                                        *m_pic_data );
@@ -218,7 +219,6 @@ void ModeDecider::DoLevelDecn( int level )
     if (level == 2)
     {
         m_me_data_set[2]->MBSplit()[m_ymb_loc][m_xmb_loc] = 2;
-        m_me_data_set[2]->MBCommonMode()[m_ymb_loc][m_xmb_loc] = false;
         m_me_data_set[2]->MBCosts()[m_ymb_loc][m_xmb_loc] = MB_cost;
     }
 
@@ -227,11 +227,6 @@ void ModeDecider::DoLevelDecn( int level )
         m_me_data_set[2]->MBCosts()[m_ymb_loc][m_xmb_loc] = MB_cost;
         m_me_data_set[2]->MBSplit()[m_ymb_loc][m_xmb_loc] = level;
         
-        if (level==1)
-            m_me_data_set[2]->MBCommonMode()[m_ymb_loc][m_xmb_loc] = false;
-        else// level is 0
-            m_me_data_set[2]->MBCommonMode()[m_ymb_loc][m_xmb_loc] = true;
-
         // Parameters of the base-level blocks corresponding to each
         // prediction unit
         int xblock_start;
@@ -266,48 +261,6 @@ void ModeDecider::DoLevelDecn( int level )
 
     }
 
-    //     Case 2: prediction modes are all the same
-
-    PredMode predmode;
-
-    MB_cost = DoCommonMode( predmode , level );
-
-    if ( MB_cost <= m_me_data_set[2]->MBCosts()[m_ymb_loc][m_xmb_loc] )
-    {
-        m_me_data_set[2]->MBCosts()[m_ymb_loc][m_xmb_loc] = MB_cost;
-        m_me_data_set[2]->MBSplit()[m_ymb_loc][m_xmb_loc] = level;
-        m_me_data_set[2]->MBCommonMode()[m_ymb_loc][m_xmb_loc] = true;
-        // Parameters of the base-level blocks corresponding to each
-        // prediction unit
-        int xblock_start;
-        int yblock_start;
-        int xblock_end;
-        int yblock_end;
-
-        for ( int j=ystart ; j<yend ; ++j )
-        {
-            yblock_start = j<<(2-level);
-            yblock_end = (j+1)<<(2-level);
-            for ( int i=xstart ; i<xend ; ++i )
-            {
-                xblock_start = i<<(2-level);
-                xblock_end = (i+1)<<(2-level);
-                for ( int v=yblock_start ; v<yblock_end ; ++v )
-                {
-                    for ( int u=xblock_start ; u<xblock_end ; ++u )
-                    {
-                        m_me_data_set[2]->Vectors(1)[v][u] = m_me_data_set[level]->Vectors(1)[j][i];
-                        m_me_data_set[2]->Mode()[v][u] = predmode;
-                        m_me_data_set[2]->DC( Y_COMP )[v][u] = m_me_data_set[level]->DC( Y_COMP )[j][i];
-                        if ( num_refs>1 )
-                            m_me_data_set[2]->Vectors(2)[v][u] = m_me_data_set[level]->Vectors(2)[j][i];
-
-                    }// u
-                }// v
- 
-            }// i
-        }// j
-    }
 }
 
 
@@ -409,14 +362,15 @@ float ModeDecider::DoUnitDecn(const int xpos , const int ypos , const int level
     MEData& me_data = *( m_me_data_set[level] );
 
     // Coords of the top-leftmost block belonging to this unit
-    const int xblock = xpos<<(2-level);
-    const int yblock = ypos<<(2-level);
+//    const int xblock = xpos<<(2-level);
+//    const int yblock = ypos<<(2-level);
 
     const float loc_lambda = me_data.LambdaMap()[ypos][xpos];
 
     float unit_cost;
-    float mode_cost;
+    float mode_cost(0.0);
     float min_unit_cost;
+    float best_SAD_value;
  
     BlockDiffParams dparams;
 
@@ -425,44 +379,30 @@ float ModeDecider::DoUnitDecn(const int xpos , const int ypos , const int level
      // First check REF1 costs //
     /**************************/
 
-    mode_cost = ModeCost( xblock , yblock , REF1_ONLY)*m_mode_factor[level];
+//    mode_cost = ModeCost( xblock , yblock )*m_mode_factor[level];
     me_data.Mode()[ypos][xpos] = REF1_ONLY;
     me_data.PredCosts(1)[ypos][xpos].total *= m_level_factor[level];
     min_unit_cost = me_data.PredCosts(1)[ypos][xpos].total + mode_cost;
-
-    // Calculate the cost if we were to code the block as intra //
-    /************************************************************/
-
-    mode_cost = ModeCost( xblock , yblock , INTRA) * m_mode_factor[level];
-    me_data.IntraCosts()[ypos][xpos] = m_intradiff->Diff( dparams , me_data.DC( Y_COMP )[ypos][xpos] );
-    me_data.IntraCosts()[ypos][xpos] += loc_lambda * 
-                                       GetDCVar( me_data.DC( Y_COMP )[ypos][xpos] , GetDCPred( xblock , yblock ) );
-    me_data.IntraCosts()[ypos][xpos] *= m_level_factor[level];
-    unit_cost = me_data.IntraCosts()[ypos][xpos] +  mode_cost;
-
-    if ( unit_cost<min_unit_cost )
-    {
-        me_data.Mode()[ypos][xpos] = INTRA;
-        min_unit_cost = unit_cost;
-    }
+    best_SAD_value = me_data.PredCosts(1)[ypos][xpos].SAD;
 
     if (num_refs>1)
     {
        // Next check REF2 costs //
        /*************************/
 
-        mode_cost = ModeCost( xblock , yblock , REF2_ONLY)*m_mode_factor[level];
+//        mode_cost = ModeCost( xblock , yblock )*m_mode_factor[level];
         me_data.PredCosts(2)[ypos][xpos].total *= m_level_factor[level];
         unit_cost = me_data.PredCosts(2)[ypos][xpos].total + mode_cost;
         if ( unit_cost<min_unit_cost )
         {
             me_data.Mode()[ypos][xpos] = REF2_ONLY;
             min_unit_cost = unit_cost;
+            best_SAD_value = me_data.PredCosts(2)[ypos][xpos].SAD;
         }
 
-        // Finally, calculate the cost if we were to use bi-predictions //
+        // Calculate the cost if we were to use bi-predictions //
         /****************************************************************/
-        mode_cost = ModeCost( xpos , ypos , REF1AND2 )*m_mode_factor[level];
+//        mode_cost = ModeCost( xpos , ypos )*m_mode_factor[level];
 
         me_data.BiPredCosts()[ypos][xpos].mvcost =
                                        me_data.PredCosts(1)[ypos][xpos].mvcost+
@@ -481,62 +421,31 @@ float ModeDecider::DoUnitDecn(const int xpos , const int ypos , const int level
         {
             me_data.Mode()[ypos][xpos] = REF1AND2;
             min_unit_cost = unit_cost;
+            best_SAD_value = me_data.BiPredCosts()[ypos][xpos].SAD;
         }
 
     }
 
-    return min_unit_cost;
-}
+    // Calculate the cost if we were to code the block as intra //
+    /************************************************************/
 
-float ModeDecider::DoCommonMode( PredMode& predmode , const int level)
-{
-    // For a given level, examine the costs in the constituent
-    // prediction units of the MB at that level and decide 
-    // whether there should be a common prediction mode or not.
-
-    const MEData& me_data = *( m_me_data_set[level] );
-
-    // The total cost for the MB for each possible prediction mode
-    OneDArray<float> MB_cost(4);
-    for ( int i=0 ; i<4 ; ++i)
-        MB_cost[i] = ModeCost( m_xmb_loc<<2 , m_ymb_loc<<2 , PredMode(i) )*m_mode_factor[0];
-
-    // The limits of the prediction units
-    const int xstart = m_xmb_loc <<level;
-    const int ystart = m_ymb_loc <<level;
-
-    const int xend = xstart + (1<<level);
-    const int yend = ystart + (1<<level);
-
-    for (int j=ystart ; j<yend ; ++j)
+    if (best_SAD_value> 4.0*m_encparams.LumaBParams( level ).Xblen()*m_encparams.LumaBParams( level ).Yblen() )
     {
-        for (int i=xstart ; i<xend ; ++i)
+//        mode_cost = ModeCost( xblock , yblock ) * m_mode_factor[level];
+        me_data.IntraCosts()[ypos][xpos] = m_intradiff->Diff( dparams , me_data.DC( Y_COMP )[ypos][xpos] );
+//        me_data.IntraCosts()[ypos][xpos] += loc_lambda * 
+//                                       GetDCVar( me_data.DC( Y_COMP )[ypos][xpos] , GetDCPred( xblock , yblock ) );
+        me_data.IntraCosts()[ypos][xpos] *= m_level_factor[level];
+        unit_cost = me_data.IntraCosts()[ypos][xpos] +  mode_cost;
+
+        if ( unit_cost<min_unit_cost && me_data.IntraCosts()[ypos][xpos]<0.9*best_SAD_value)
         {
-            MB_cost[INTRA] += me_data.IntraCosts()[j][i];
-            MB_cost[REF1_ONLY] += me_data.PredCosts(1)[j][i].total;
-            if ( num_refs>1 )
-            {
-                MB_cost[REF2_ONLY] += me_data.PredCosts(2)[j][i].total;
-                MB_cost[REF1AND2] += me_data.BiPredCosts()[j][i].total;
-            }
-        }// i
-    }// i
-
-
-    // Find the minimum
-    predmode = INTRA;
-    if ( MB_cost[REF1_ONLY]<MB_cost[predmode] )
-        predmode = REF1_ONLY;
-
-    if ( num_refs>1)
-    {
-        if ( MB_cost[REF2_ONLY]<MB_cost[predmode] )
-            predmode = REF2_ONLY;
-        if ( MB_cost[REF1AND2]<MB_cost[predmode] )
-            predmode = REF1AND2;
+            me_data.Mode()[ypos][xpos] = INTRA;
+            min_unit_cost = unit_cost;
+        }
     }
- 
-    return MB_cost[predmode];
+
+    return min_unit_cost;
 }
 
 ValueType ModeDecider::GetDCPred( int xblock , int yblock )
@@ -556,8 +465,7 @@ ValueType ModeDecider::GetDCPred( int xblock , int yblock )
     return dc_pred;
 }
 
-float ModeDecider::ModeCost(const int xindex , const int yindex , 
-                 const PredMode predmode )
+float ModeDecider::ModeCost(const int xindex , const int yindex)
 {
     // Computes the variation of the given mode, predmode, from its immediate neighbours
     // First, get a prediction for the mode
