@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: me_subpel.cpp,v 1.13 2005/09/27 12:59:34 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: me_subpel.cpp,v 1.14 2006/04/20 10:41:58 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -69,7 +69,7 @@ void SubpelRefine::DoSubpel(const FrameBuffer& my_buffer,int frame_num, MEData&
 
     const FrameSort fsort = my_buffer.GetFrame(frame_num).GetFparams().FSort();
 
-    if (fsort != I_frame)
+    if (fsort.IsInter())
     {
         // Get the references
         const vector<int>& refs = my_buffer.GetFrame(frame_num).GetFparams().Refs();
