@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: me_utils.cpp,v 1.16 2005/10/07 10:17:44 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: me_utils.cpp,v 1.19 2006/06/05 14:54:17 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -20,7 +20,9 @@
 * Portions created by the Initial Developer are Copyright (C) 2004.
 * All Rights Reserved.
 *
-* Contributor(s): Thomas Davies (Original Author), Peter Meerwald (pmeerw@users.sourceforge.net)
+* Contributor(s): Thomas Davies (Original Author), 
+*                 Peter Meerwald (pmeerw@users.sourceforge.net)
+*                 Steve Bearcroft (bearcrsw@users.sourceforge.net)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -131,6 +133,10 @@ BiBlockEighthPel::BiBlockEighthPel( const PicArray& ref1 , const PicArray& ref2
 
 float PelBlockDiff::Diff( const BlockDiffParams& dparams, const MVector& mv )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return 0;
+    }
 
     CalcValueType sum( 0 );
 
@@ -150,15 +156,15 @@ float PelBlockDiff::Diff( const BlockDiffParams& dparams, const MVector& mv )
 #if defined(HAVE_MMX)
         return static_cast<float>(simple_block_diff_mmx_4(dparams, mv, m_pic_data, m_ref_data, INT_MAX));
 #else
-    	ValueType diff;    
-        for ( int j=dparams.Yp() ; j!=dparams.Yp()+dparams.Yl() ; ++j )
+        ValueType diff;    
+        for ( int j=dparams.Yp() ; j<dparams.Yp()+dparams.Yl() ; ++j )
         { 
-        	for(int i=dparams.Xp() ; i!= dparams.Xp()+dparams.Xl() ; ++i )
-        	{
-            	diff = m_pic_data[j][i]-m_ref_data[j+mv.y][i+mv.x];
-            	sum += std::abs( diff );
-        	}// i
-    	}// j
+            for(int i=dparams.Xp() ; i< dparams.Xp()+dparams.Xl() ; ++i )
+            {
+                diff = m_pic_data[j][i]-m_ref_data[j+mv.y][i+mv.x];
+                sum += std::abs( diff );
+            }// i
+        }// j
 #endif /* HAVE_MMX */
     }
     else
@@ -166,10 +172,10 @@ float PelBlockDiff::Diff( const BlockDiffParams& dparams, const MVector& mv )
 #if defined (HAVE_MMX)
         return static_cast<float>(bchk_simple_block_diff_mmx_4(dparams, mv, m_pic_data, m_ref_data, INT_MAX));
 #else
-    	ValueType diff;    
-        for ( int j=dparams.Yp() ; j!=dparams.Yp()+dparams.Yl() ; ++j )
+        ValueType diff;    
+        for ( int j=dparams.Yp() ; j < dparams.Yp()+dparams.Yl() ; ++j )
         { 
-            for( int i=dparams.Xp() ; i!=dparams.Xp()+dparams.Xl() ; ++i )
+            for( int i=dparams.Xp() ; i < dparams.Xp()+dparams.Xl() ; ++i )
             {
                 diff = m_pic_data[j][i] - m_ref_data[BChk(j+mv.y , m_ref_data.LengthY())][BChk(i+mv.x , m_ref_data.LengthX())];
                 sum += std::abs( diff );
@@ -188,6 +194,10 @@ void PelBlockDiff::Diff( const BlockDiffParams& dparams,
                          float& best_sum,
                          MVector& best_mv )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return;
+    }
 
     CalcValueType sum( 0 );
 
@@ -206,16 +216,16 @@ void PelBlockDiff::Diff( const BlockDiffParams& dparams,
     {
 #if defined (HAVE_MMX)
         sum  = simple_block_diff_mmx_4(dparams, mv, m_pic_data, m_ref_data, static_cast<int>(best_sum));
-	    if (sum < best_sum)
-	    {
-   		    best_sum = sum;
-   			best_mv = mv;
-	    }
-	    return; 
+        if (sum < best_sum)
+        {
+               best_sum = sum;
+               best_mv = mv;
+        }
+        return; 
 #else
-    	ValueType diff;    
-    	ValueType *pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
-    	const int pic_next( m_pic_data.LengthX() - dparams.Xl() ); // - go down a row and back along
+        ValueType diff;    
+        ValueType *pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
+        const int pic_next( m_pic_data.LengthX() - dparams.Xl() ); // - go down a row and back along
     
 
         ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
@@ -237,15 +247,15 @@ void PelBlockDiff::Diff( const BlockDiffParams& dparams,
     else
     {
 #if defined (HAVE_MMX)
-       	sum = (bchk_simple_block_diff_mmx_4(dparams, mv, m_pic_data, m_ref_data, static_cast<int>(best_sum)));
-		if (sum < best_sum)
-		{
-			best_sum = sum;
-			best_mv = mv;
-		}
-		return;
+           sum = (bchk_simple_block_diff_mmx_4(dparams, mv, m_pic_data, m_ref_data, static_cast<int>(best_sum)));
+        if (sum < best_sum)
+        {
+            best_sum = sum;
+            best_mv = mv;
+        }
+        return;
 #else
-    	ValueType diff;    
+        ValueType diff;    
         for ( int j=dparams.Yp() ; j<dparams.Yend() ; ++j )
         { 
             for( int i=dparams.Xp() ; i<dparams.Xend() ; ++i )
@@ -269,58 +279,60 @@ void PelBlockDiff::Diff( const BlockDiffParams& dparams,
 
 float IntraBlockDiff::Diff( const BlockDiffParams& dparams , ValueType& dc_val )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        dc_val = 0;
+        return 0;
+    }
 
      //computes the cost if block is predicted by its dc component
 #if defined(HAVE_MMX)
-	CalcValueType intra_cost =
-			simple_intra_block_diff_mmx_4 (dparams, m_pic_data, dc_val);
+    CalcValueType intra_cost =
+            simple_intra_block_diff_mmx_4 (dparams, m_pic_data, dc_val);
 
 #ifdef DIRAC_DEBUG
     CalcValueType int_dc( 0 );
-	ValueType non_mmx_dc(0);
+    ValueType non_mmx_dc(0);
 
-    for ( int j=dparams.Yp() ; j!=dparams.Yp()+dparams.Yl() ; ++j)
-        for(int i=dparams.Xp(); i!=dparams.Xp()+dparams.Xl() ; ++i )
+    for ( int j=dparams.Yp() ; j<dparams.Yp()+dparams.Yl() ; ++j)
+        for(int i=dparams.Xp(); i<dparams.Xp()+dparams.Xl() ; ++i )
             int_dc += static_cast<int>( m_pic_data[j][i] );
 
     int_dc /= ( dparams.Xl() * dparams.Yl() );
 
-    // Just give dc to 8-bit accuracy
-    non_mmx_dc = static_cast<ValueType>( (int_dc+2)>>2 );
+    non_mmx_dc = static_cast<ValueType>( int_dc );
 
     // Now compute the resulting SAD
-    ValueType dc( non_mmx_dc<<2 );
+    ValueType dc( non_mmx_dc );
     CalcValueType non_mmx_intra_cost( 0 );
 
     for (int j=dparams.Yp(); j<dparams.Yend() ; ++j)
         for( int i=dparams.Xp() ; i<dparams.Xend() ;++i )
             non_mmx_intra_cost += std::abs( m_pic_data[j][i] - dc );
 
-	if (non_mmx_dc != dc_val || non_mmx_intra_cost != intra_cost)
-	{
-		std::cerr << "MMX vals: dc=" << dc_val;
-		std::cerr << " cost=" << intra_cost << std::endl;
-		//print_arr (pic_data, width[i%5], height[i%5]);
-		std::cerr << "non-MMX vals: dc=" << non_mmx_dc;
-		std::cerr << " cost=" << non_mmx_intra_cost << std::endl;
-	}
+    if (non_mmx_dc != dc_val || non_mmx_intra_cost != intra_cost)
+    {
+        std::cerr << "MMX vals: dc=" << dc_val;
+        std::cerr << " cost=" << intra_cost << std::endl;
+        //print_arr (pic_data, width[i%5], height[i%5]);
+        std::cerr << "non-MMX vals: dc=" << non_mmx_dc;
+        std::cerr << " cost=" << non_mmx_intra_cost << std::endl;
+    }
 #endif
     return static_cast<float>( intra_cost );
 #else
-
     CalcValueType int_dc( 0 );
 
-    for ( int j=dparams.Yp() ; j!=dparams.Yp()+dparams.Yl() ; ++j)
-        for(int i=dparams.Xp(); i!=dparams.Xp()+dparams.Xl() ; ++i )
+    for ( int j=dparams.Yp() ; j<dparams.Yp()+dparams.Yl() ; ++j)
+        for(int i=dparams.Xp(); i<dparams.Xp()+dparams.Xl() ; ++i )
             int_dc += static_cast<int>( m_pic_data[j][i] );
 
     int_dc /= ( dparams.Xl() * dparams.Yl() );
 
-    // Just give dc to 8-bit accuracy
-    dc_val = static_cast<ValueType>( (int_dc+2)>>2 );
+    dc_val = static_cast<ValueType>( int_dc );
 
     // Now compute the resulting SAD
-    ValueType dc( dc_val<<2 );
+    ValueType dc( dc_val );
     CalcValueType intra_cost( 0 );
 
     for (int j=dparams.Yp(); j<dparams.Yend() ; ++j)
@@ -335,6 +347,10 @@ float IntraBlockDiff::Diff( const BlockDiffParams& dparams , ValueType& dc_val )
 float BlockDiffHalfPel::Diff(  const BlockDiffParams& dparams , 
                                       const MVector& mv )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return 0;
+    }
    //Where to start in the upconverted image
     const ImageCoords ref_start( ( dparams.Xp()<<1 ) + mv.x ,( dparams.Yp()<<1 ) + mv.y );
     const ImageCoords ref_stop( ref_start.x+(dparams.Xl()<<1) , ref_start.y+(dparams.Yl()<<1));
@@ -353,18 +369,18 @@ float BlockDiffHalfPel::Diff(  const BlockDiffParams& dparams ,
     if ( !bounds_check )
     {
 #if defined (HAVE_MMX)
-		MVector rmdr(0,0);
-		const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
-		const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
-		sum = simple_block_diff_up_mmx_4 (m_pic_data, m_ref_data,
-										start_pos, end_pos,
-										ref_start, ref_stop,
-										rmdr,
-										sum,
-										static_cast<float>(INT_MAX));
+        MVector rmdr(0,0);
+        const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
+        const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
+        sum = simple_block_diff_up_mmx_4 (m_pic_data, m_ref_data,
+                                        start_pos, end_pos,
+                                        ref_start, ref_stop,
+                                        rmdr,
+                                        sum,
+                                        static_cast<float>(INT_MAX));
 #else
-    	ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
-    	const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
+        ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
+        const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
         ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
         const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
 
@@ -381,8 +397,8 @@ float BlockDiffHalfPel::Diff(  const BlockDiffParams& dparams ,
     else
     {
         // We're doing bounds checking because we'll fall off the edge of the reference otherwise.
-    	ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
-    	const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
+        ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
+        const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
         for( int y=dparams.Yl(), ry=ref_start.y, by=BChk(ry,m_ref_data.LengthY()); 
              y>0; 
              --y, pic_curr+=pic_next, ry+=2 , by=BChk(ry,m_ref_data.LengthY()))
@@ -408,6 +424,10 @@ void BlockDiffHalfPel::Diff( const BlockDiffParams& dparams,
                                    MvCostData& best_costs ,
                                    MVector& best_mv )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return; 
+    }
 
     //Where to start in the upconverted image
     const ImageCoords ref_start( ( dparams.Xp()<<1 ) + mv.x ,( dparams.Yp()<<1 ) + mv.y );
@@ -428,21 +448,21 @@ void BlockDiffHalfPel::Diff( const BlockDiffParams& dparams,
     if ( !bounds_check )
     {
 #if defined (HAVE_MMX)
-		
-		const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
-		const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
-		MVector rmdr(0,0);
-		sum = simple_block_diff_up_mmx_4 (m_pic_data, m_ref_data,
-										start_pos, end_pos,
-										ref_start, ref_stop,
-										rmdr,
-										sum,
-										best_costs.total);
+        
+        const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
+        const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
+        MVector rmdr(0,0);
+        sum = simple_block_diff_up_mmx_4 (m_pic_data, m_ref_data,
+                                        start_pos, end_pos,
+                                        ref_start, ref_stop,
+                                        rmdr,
+                                        sum,
+                                        best_costs.total);
         if ( sum>=best_costs.total )
             return;
 #else
-   		ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
-   		const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
+           ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
+           const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
         ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
         const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
 
@@ -462,8 +482,8 @@ void BlockDiffHalfPel::Diff( const BlockDiffParams& dparams,
     else
     {
         // We're doing bounds checking because we'll fall off the edge of the reference otherwise.
-   		ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
-   		const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
+           ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
+           const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
         for( int y=dparams.Yl(), ry=ref_start.y, by=BChk(ry,m_ref_data.LengthY()); 
              y>0; 
              --y, pic_curr+=pic_next, ry+=2 , by=BChk(ry,m_ref_data.LengthY()))
@@ -490,6 +510,10 @@ void BlockDiffHalfPel::Diff( const BlockDiffParams& dparams,
 
 float BlockDiffQuarterPel::Diff(  const BlockDiffParams& dparams , const MVector& mv )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return 0; 
+    }
    // Set up the start point in the reference image by rounding the motion vector
     // to 1/2 pel accuracy.NB: bit shift rounds negative values DOWN, as required
     const MVector roundvec( mv.x>>1 , mv.y>>1 );
@@ -509,29 +533,29 @@ float BlockDiffQuarterPel::Diff(  const BlockDiffParams& dparams , const MVector
         bounds_check = true;
 
     float sum( 0.0f );
-   	CalcValueType temp;
+       CalcValueType temp;
 
 
     if ( !bounds_check )
     {
 #if defined (HAVE_MMX)
-		const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
-		const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
+        const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
+        const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
            
-		sum = simple_block_diff_up_mmx_4 (m_pic_data, m_ref_data,
-										start_pos, end_pos,
-										ref_start, ref_stop,
-										rmdr,
-										sum,
-										static_cast<float>(INT_MAX));
+        sum = simple_block_diff_up_mmx_4 (m_pic_data, m_ref_data,
+                                        start_pos, end_pos,
+                                        ref_start, ref_stop,
+                                        rmdr,
+                                        sum,
+                                        static_cast<float>(INT_MAX));
 
 #else
-	    ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
+        ValueType* pic_curr = &m_pic_data[dparams.Yp()][dparams.Xp()];
         const int pic_next( m_pic_data.LengthX() - dparams.Xl() );// go down a row and back up
         if( rmdr.x == 0 && rmdr.y == 0 )
         {
-        	ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
-        	const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
+            ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
+            const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
             for( int y=dparams.Yl(); y > 0; --y, pic_curr+=pic_next, ref_curr+=ref_next )
             {
                 for( int x=dparams.Xl(); x > 0; --x, ++pic_curr, ref_curr+=2 )
@@ -542,8 +566,8 @@ float BlockDiffQuarterPel::Diff(  const BlockDiffParams& dparams , const MVector
         }
         else if( rmdr.y == 0 )
         {
-        	ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
-        	const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
+            ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
+            const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
             for( int y=dparams.Yl(); y > 0; --y, pic_curr+=pic_next, ref_curr+=ref_next )
             {
                 for( int x=dparams.Xl(); x > 0; --x, ++pic_curr, ref_curr+=2 )
@@ -558,8 +582,8 @@ float BlockDiffQuarterPel::Diff(  const BlockDiffParams& dparams , const MVector
         }
         else if( rmdr.x == 0 )
         {
-        	ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
-        	const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
+            ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
+            const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
             for( int y=dparams.Yl(); y > 0; --y, pic_curr+=pic_next, ref_curr+=ref_next )
             {
                 for( int x=dparams.Xl(); x > 0; --x, ++pic_curr, ref_curr+=2 )
@@ -574,8 +598,8 @@ float BlockDiffQuarterPel::Diff(  const BlockDiffParams& dparams , const MVector
         }
         else
         {
-        	ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
-        	const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
+            ValueType *ref_curr = &m_ref_data[ref_start.y][ref_start.x];
+            const int ref_next( (m_ref_data.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
             for( int y=dparams.Yl(); y > 0; --y, pic_curr+=pic_next, ref_curr+=ref_next )
             {
                 for( int x=dparams.Xl(); x > 0; --x, ++pic_curr, ref_curr+=2 )
@@ -635,6 +659,10 @@ void BlockDiffQuarterPel::Diff( const BlockDiffParams& dparams,
                                    MvCostData& best_costs ,
                                    MVector& best_mv)
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return; 
+    }
 
     // Set up the start point in the reference image by rounding the motion vector
     // to 1/2 pel accuracy.NB: bit shift rounds negative values DOWN, as required
@@ -663,15 +691,15 @@ void BlockDiffQuarterPel::Diff( const BlockDiffParams& dparams,
     if ( !bounds_check )
     {
 #if defined (HAVE_MMX)
-		const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
-		const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
+        const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
+        const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
         
-		sum = simple_block_diff_up_mmx_4 (m_pic_data, m_ref_data,
-										start_pos, end_pos,
-										ref_start, ref_stop,
-										rmdr,
-										sum,
-										best_costs.total);
+        sum = simple_block_diff_up_mmx_4 (m_pic_data, m_ref_data,
+                                        start_pos, end_pos,
+                                        ref_start, ref_stop,
+                                        rmdr,
+                                        sum,
+                                        best_costs.total);
 
         if ( sum>=best_costs.total )
             return;
@@ -765,11 +793,11 @@ void BlockDiffQuarterPel::Diff( const BlockDiffParams& dparams,
         const int refXlen( m_ref_data.LengthX() );
         const int refYlen( m_ref_data.LengthY() );
 
-        for(int y = dparams.Yl(), uY = ref_start.y,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen);
-            y > 0; --y, uY += 2,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen))
+        for(int y = dparams.Yp(), uY = ref_start.y,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen);
+            y < dparams.Yend(); ++y, uY += 2,BuY=BChk(uY,refYlen),BuY1=BChk(uY+1,refYlen))
         {
-            for(int x = dparams.Xl(), uX = ref_start.x,BuX=BChk(uX,refXlen),BuX1=BChk(uX+1,refXlen);
-                x > 0 ; --x, uX += 2,BuX=BChk(uX,refXlen),BuX1=BChk(uX+1,refXlen))
+            for(int x = dparams.Xp(), uX = ref_start.x,BuX=BChk(uX,refXlen),BuX1=BChk(uX+1,refXlen);
+                x < dparams.Xend(); ++x, uX += 2,BuX=BChk(uX,refXlen),BuX1=BChk(uX+1,refXlen))
             {
  
                 temp = (     linear_wts[0] * CalcValueType( m_ref_data[BuY][BuX] ) +
@@ -798,6 +826,10 @@ void BlockDiffQuarterPel::Diff( const BlockDiffParams& dparams,
 
 float BlockDiffEighthPel::Diff(  const BlockDiffParams& dparams , const MVector& mv )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return 0; 
+    }
    //Set up the start point in the reference image by rounding the motion vector
     //NB: bit shift rounds negative values DOWN, as required
     const MVector roundvec( mv.x>>2 , mv.y>>2 );
@@ -925,6 +957,10 @@ void BlockDiffEighthPel::Diff( const BlockDiffParams& dparams,
                                    MvCostData& best_costs ,
                                    MVector& best_mv)
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return; 
+    }
     //Set up the start point in the reference image by rounding the motion vector
     //NB: bit shift rounds negative values DOWN, as required
     const MVector roundvec( mv.x>>2 , mv.y>>2 );
@@ -1074,6 +1110,10 @@ float BiBlockHalfPel::Diff(  const BlockDiffParams& dparams ,
                              const MVector& mv1 ,
                              const MVector& mv2 )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return 0; 
+    }
     // First create a difference array, and subtract the reference 1 data into it
     TwoDArray<ValueType> diff_array( dparams.Yl() , dparams.Xl() );
 
@@ -1186,6 +1226,10 @@ float BiBlockQuarterPel::Diff(  const BlockDiffParams& dparams ,
                              const MVector& mv1 ,
                              const MVector& mv2 )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return 0; 
+    }
     // First create a difference array, and subtract the reference 1 data into it
     TwoDArray<ValueType> diff_array( dparams.Yl() , dparams.Xl() );
 
@@ -1223,13 +1267,13 @@ float BiBlockQuarterPel::Diff(  const BlockDiffParams& dparams ,
     if ( !bounds_check )
     {
 #if defined (HAVE_MMX)
-		const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
-		const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
+        const ImageCoords start_pos(dparams.Xp(), dparams.Yp());
+        const ImageCoords end_pos(dparams.Xp() + dparams.Xl(), dparams.Yp() + dparams.Yl());
         
-		simple_biblock_diff_pic_mmx_4 (m_pic_data, m_ref_data1, diff_array,
-									start_pos, end_pos,
-									ref_start1, ref_stop1,
-									rmdr1);
+        simple_biblock_diff_pic_mmx_4 (m_pic_data, m_ref_data1, diff_array,
+                                    start_pos, end_pos,
+                                    ref_start1, ref_stop1,
+                                    rmdr1);
 #else
         ValueType *ref_curr = &m_ref_data1[ref_start1.y][ref_start1.x];
         const int ref_next( (m_ref_data1.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
@@ -1340,10 +1384,10 @@ float BiBlockQuarterPel::Diff(  const BlockDiffParams& dparams ,
 
 
 #if defined (HAVE_MMX)
-			sum = static_cast<float>( simple_biblock_diff_up_mmx_4 (diff_array, 
-												m_ref_data2,
-												ref_start2, ref_stop2,
-												rmdr2) );
+            sum = static_cast<float>( simple_biblock_diff_up_mmx_4 (diff_array, 
+                                                m_ref_data2,
+                                                ref_start2, ref_stop2,
+                                                rmdr2) );
 #else
         ValueType *ref_curr = &m_ref_data2[ref_start2.y][ref_start2.x];
         const int ref_next( (m_ref_data2.LengthX() - dparams.Xl())*2 );// go down 2 rows and back up
@@ -1412,11 +1456,11 @@ float BiBlockQuarterPel::Diff(  const BlockDiffParams& dparams ,
                                            rmdr2.x * rmdr2.y };              //br
 
         // We're doing bounds checking because we'll fall off the edge of the reference otherwise.
-        for( int y=dparams.Yl(), ry=ref_start1.y, by=BChk(ry,m_ref_data2.LengthY()),by1=BChk(ry+1,m_ref_data2.LengthY()); 
+        for( int y=dparams.Yl(), ry=ref_start2.y, by=BChk(ry,m_ref_data2.LengthY()),by1=BChk(ry+1,m_ref_data2.LengthY()); 
              y>0; 
              --y, pic_curr+=pic_next, ry+=2 , by=BChk(ry,m_ref_data2.LengthY()),by1=BChk(ry+1,m_ref_data2.LengthY()))
         {
-             for( int x=dparams.Xl() , rx=ref_start1.x , bx=BChk(rx,m_ref_data2.LengthX()), bx1=BChk(rx+1,m_ref_data2.LengthX()); 
+             for( int x=dparams.Xl() , rx=ref_start2.x , bx=BChk(rx,m_ref_data2.LengthX()), bx1=BChk(rx+1,m_ref_data2.LengthX()); 
                   x>0 ; 
                   --x, ++pic_curr, rx+=2 , ++diff_curr, bx=BChk(rx,m_ref_data2.LengthX()), bx1=BChk(rx+1,m_ref_data2.LengthX()))
              {
@@ -1439,6 +1483,10 @@ float BiBlockEighthPel::Diff(  const BlockDiffParams& dparams ,
                              const MVector& mv1 ,
                              const MVector& mv2 )
 {
+    if (dparams.Xl() <= 0 || dparams.Yl() <= 0)
+    {
+        return 0; 
+    }
 
     // First create a difference array, and subtract the reference 1 data into it
     TwoDArray<ValueType> diff_array( dparams.Yl() , dparams.Xl() );
