@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: motion_estimate.cpp,v 1.16 2005/09/27 13:07:00 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: motion_estimate.cpp,v 1.19 2007/04/03 13:02:38 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -66,19 +66,46 @@ void MotionEstimator::DoME(const FrameBuffer& my_buffer, int frame_num, MEData&
     const std::vector<int>& refs = my_buffer.GetFrame(frame_num).GetFparams().Refs();
 
     const int num_refs = refs.size();
-    if ( fparams.FSort() == L1_frame )
-        lambda = m_encparams.L1MELambda();
-    else
+    if ( fparams.IsBFrame())
         lambda = m_encparams.L2MELambda();
+    else
+        lambda = m_encparams.L1MELambda();
 
     // Set up the lambda to be used
     me_data.SetLambdaMap( num_refs , lambda );
 
+    MVPrecisionType orig_prec = m_encparams.MVPrecision();
+
     // Step 2. 
     // Pixel accurate vectors are then refined to sub-pixel accuracy
 
-    SubpelRefine pelrefine( m_encparams );
-    pelrefine.DoSubpel( my_buffer , frame_num , me_data );
+    if (orig_prec != MV_PRECISION_PIXEL)
+    {
+        SubpelRefine pelrefine( m_encparams );
+        pelrefine.DoSubpel( my_buffer , frame_num , me_data );
+    }
+    else
+    {
+        // FIXME: HACK HACK
+        // Mutiplying the motion vectors by 2 and setting MV precision to
+        // HALF_PIXEL to implement pixel accurate motion estimate
+        MvArray &mv_arr1 = me_data.Vectors(1);
+        for (int j = 0; j < mv_arr1.LengthY(); ++j)
+        {
+            for (int i = 0; i < mv_arr1.LengthX(); ++i)
+                mv_arr1[j][i] = mv_arr1[j][i] << 1;
+        }
+        if (num_refs > 1)
+        {
+            MvArray &mv_arr2 = me_data.Vectors(2);
+            for (int j = 0; j < mv_arr2.LengthY(); ++j)
+            {
+                for (int i = 0; i < mv_arr2.LengthX(); ++i)
+                    mv_arr2[j][i] = mv_arr2[j][i] << 1;
+            }
+        }
+        m_encparams.SetMVPrecision(MV_PRECISION_HALF_PIXEL);
+    }
 
     // Step3.
     // We now have to decide how each macroblock should be split 
@@ -86,13 +113,36 @@ void MotionEstimator::DoME(const FrameBuffer& my_buffer, int frame_num, MEData&
 
     ModeDecider my_mode_dec( m_encparams );
     my_mode_dec.DoModeDecn( my_buffer , frame_num , me_data );
+    
+    if (orig_prec ==  MV_PRECISION_PIXEL)
+    {
+        // FIXME: HACK HACK
+        // Divide the motion vectors by 2 to convert back to pixel
+        // accurate motion vectors and reset MV precision to
+        // PIXEL accuracy 
+        MvArray &mv_arr1 = me_data.Vectors(1);
+        for (int j = 0; j < mv_arr1.LengthY(); ++j)
+        {
+            for (int i = 0; i < mv_arr1.LengthX(); ++i)
+                mv_arr1[j][i] = mv_arr1[j][i] >> 1;
+        }
+        if (num_refs > 1)
+        {
+            MvArray &mv_arr2 = me_data.Vectors(2);
+            for (int j = 0; j < mv_arr2.LengthY(); ++j)
+            {
+                for (int i = 0; i < mv_arr2.LengthX(); ++i)
+                    mv_arr2[j][i] = mv_arr2[j][i]>>1;
+            }
+        }
+        m_encparams.SetMVPrecision(MV_PRECISION_PIXEL);
+    }
 
     // Finally, although not strictly part of motion estimation,
     // we have to assign DC values for chroma components for
     // blocks we're decided are intra.
 
-    if (fparams.CFormat() != Yonly)
-        SetChromaDC( my_buffer , frame_num , me_data );
+    SetChromaDC( my_buffer , frame_num , me_data );
 
 //return false;
 }
