@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: pixel_match.cpp,v 1.5 2005/09/27 12:55:11 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: pixel_match.cpp,v 1.12 2007/08/02 14:22:51 tjdwave Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -20,7 +20,8 @@
 * Portions created by the Initial Developer are Copm_yright (C) 2004.
 * All Rights Reserved.
 *
-* Contributor(s): Thomas Davies (Original Author)
+* Contributor(s): Thomas Davies (Original Author),
+*                 Tim Borer
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -68,11 +69,16 @@ void PixelMatcher::DoSearch(const FrameBuffer& my_buffer, int frame_num, MEData&
 
     const vector<int>& refs = my_buffer.GetFrame( frame_num ).GetFparams().Refs();
     ref1 = refs[0];
+
     if (refs.size()>1)
         ref2 = refs[1];
     else    
         ref2 = ref1;
 
+    // Record temporal distances
+    m_tdiff[0] = std::abs( ref1 - frame_num );
+    m_tdiff[1] = std::abs( ref2 - frame_num );
+
     // Obtain C++ references to the reference picture luma components
     const PicArray& ref1_data = my_buffer.GetComponent(ref1 , Y_COMP);
     const PicArray& ref2_data = my_buffer.GetComponent(ref2 , Y_COMP);
@@ -80,59 +86,70 @@ void PixelMatcher::DoSearch(const FrameBuffer& my_buffer, int frame_num, MEData&
     // Determine the frame sort - this affects the motion estimation Lagrangian parameter
     m_fsort = my_buffer.GetFrame(frame_num).GetFparams().FSort();
 
-    // Set the number of downconversion levels - not too many or we run out of picture!    
-    m_depth = ( int) std::min( log(((double) pic_data.LengthX())/12.0)/log(2.0) , 
-                             log(((double) pic_data.LengthY())/12.0)/log(2.0) );
 
-    // These arrays will contain the downconverted picture and MvData hierarchy
-    OneDArray<PicArray*> ref1_down( Range( 1 , m_depth ) );
-    OneDArray<PicArray*> ref2_down( Range( 1 , m_depth ) );
-    OneDArray<PicArray*> pic_down( Range( 1 , m_depth ) );
-    OneDArray<MEData*> me_data_set( Range( 1 , m_depth ) );
-
-    // Populate the hierarchies
-    MakePicHierarchy( pic_data , pic_down );
-    MakePicHierarchy( ref1_data , ref1_down );
-    if (ref1 != ref2)
-        MakePicHierarchy( ref2_data , ref2_down );
-
-    MakeMEDataHierarchy( pic_down , me_data_set );
-
-     // Now do the work! //
-    //////////////////////
-
-    // Start with motion estimating at the very lowest level
-    m_level = m_depth;
-
-    MatchPic( *(pic_down[m_depth]) , *(ref1_down[m_depth]) , *(me_data_set[m_depth]) ,
-                                     *(me_data_set[m_depth]) , 1 );    
-    if ( ref1 != ref2 )
-        MatchPic( *(pic_down[m_depth]) , *(ref2_down[m_depth]) , *(me_data_set[m_depth]) , 
-                                         *(me_data_set[m_depth]) , 2 );
-
-     // Do the intervening levels - here we can have a genuine set of guide vectors
-    for ( m_level=m_depth-1 ; m_level>=1 ; --m_level )
+    if ( m_encparams.FullSearch() == false )
     {
-        MatchPic( *(pic_down[m_level]) , *(ref1_down[m_level]) , *(me_data_set[m_level]) , 
+        // Set the number of downconversion levels - not too many or we run out of picture!
+        m_depth = ( int) std::min( log(((double) pic_data.LengthX())/12.0)/log(2.0) , 
+                                 log(((double) pic_data.LengthY())/12.0)/log(2.0) );
+
+        // These arrays will contain the downconverted picture and MvData hierarchy
+        OneDArray<PicArray*> ref1_down( Range( 1 , m_depth ) );
+        OneDArray<PicArray*> ref2_down( Range( 1 , m_depth ) );
+        OneDArray<PicArray*> pic_down( Range( 1 , m_depth ) );
+        OneDArray<MEData*> me_data_set( Range( 1 , m_depth ) );
+
+        // Populate the hierarchies
+        MakePicHierarchy( pic_data , pic_down );
+        MakePicHierarchy( ref1_data , ref1_down );
+        if (ref1 != ref2)
+            MakePicHierarchy( ref2_data , ref2_down );
+
+        MakeMEDataHierarchy( pic_down , me_data_set );
+
+         // Now do the work! //
+        //////////////////////
+
+        // Start with motion estimating at the very lowest level
+        m_level = m_depth;
+
+        MatchPic( *(pic_down[m_depth]) , *(ref1_down[m_depth]) , *(me_data_set[m_depth]) ,
+                                     *(me_data_set[m_depth]) , 1 );    
+        if ( ref1 != ref2 )
+            MatchPic( *(pic_down[m_depth]) , *(ref2_down[m_depth]) , *(me_data_set[m_depth]) , 
+                                             *(me_data_set[m_depth]) , 2 );
+
+         // Do the intervening levels - here we can have a genuine set of guide vectors
+        for ( m_level=m_depth-1 ; m_level>=1 ; --m_level )
+        {
+            MatchPic( *(pic_down[m_level]) , *(ref1_down[m_level]) , *(me_data_set[m_level]) , 
                                          *(me_data_set[m_level+1]) , 1 );
-        if (ref1!=ref2)
-            MatchPic( *(pic_down[m_level]) , *(ref2_down[m_level]) , *(me_data_set[m_level]) , 
-                                             *(me_data_set[m_level+1]) , 2 );
-        
-    }// level
+            if (ref1!=ref2)
+                MatchPic( *(pic_down[m_level]) , *(ref2_down[m_level]) , *(me_data_set[m_level]) , 
+                                                 *(me_data_set[m_level+1]) , 2 );    
+        }// level
 
-    // Finally, do the top level, with the pictures themselves
-    m_level = 0;
-    MatchPic( pic_data , ref1_data, me_data , *(me_data_set[1]) , 1 );
-    if ( ref1 != ref2 )
-        MatchPic( pic_data , ref2_data , me_data , *(me_data_set[1]) , 2 );
+        // Finally, do the top level, with the pictures themselves
+        m_level = 0;
+        MatchPic( pic_data , ref1_data, me_data , *(me_data_set[1]) , 1 );
+        if ( ref1 != ref2 )
+            MatchPic( pic_data , ref2_data , me_data , *(me_data_set[1]) , 2 );
 
-    // Now we're finished, tidy everything up ...
-    TidyPics( pic_down );
-    TidyPics( ref1_down );
-    if (ref1 != ref2)
-        TidyPics( ref2_down );
-    TidyMEData( me_data_set );
+        // Now we're finished, tidy everything up ...
+        TidyPics( pic_down );
+        TidyPics( ref1_down );
+        if (ref1 != ref2)
+            TidyPics( ref2_down );
+        TidyMEData( me_data_set );
+    }
+    else
+    {
+        m_depth = 0;
+        m_level = 0;
+        MatchPic( pic_data , ref1_data, me_data , me_data , 1 );
+        if ( ref1 != ref2 )
+            MatchPic( pic_data , ref2_data , me_data , me_data , 2 );
+    }
 
 }
 
@@ -169,11 +186,17 @@ void PixelMatcher::MakeMEDataHierarchy(const OneDArray< PicArray*>& down_data,
     int xnumblocks , ynumblocks;
     const OLBParams bparams = m_encparams.LumaBParams(2);
 
+    // We might not have an integral number of Macroblocks and blocks in 
+    // a picture. So we go start of with the number of macroblocks in the
+    // full size picture and calculate the number of in the downsized pics
+    // from this.
+    xnumblocks = m_encparams.XNumBlocks();
+    ynumblocks = m_encparams.YNumBlocks();
     for (int i=1 ; i<=m_depth;++i)
     {
 
-        xnumblocks = down_data[i]->LengthX()/bparams.Xbsep();
-        ynumblocks = down_data[i]->LengthY()/bparams.Ybsep();
+        xnumblocks = xnumblocks>>1;
+        ynumblocks = ynumblocks>>1;
 
         if (( down_data[i]->LengthX() )%bparams.Xbsep() != 0)
             xnumblocks++;
@@ -181,7 +204,7 @@ void PixelMatcher::MakeMEDataHierarchy(const OneDArray< PicArray*>& down_data,
         if (( down_data[i]->LengthY() )%bparams.Ybsep() != 0)
             ynumblocks++;
 
-        me_data_set[i] = new MEData( 0 , 0 , xnumblocks , ynumblocks );
+        me_data_set[i] = new MEData( 0 , 0 , xnumblocks , ynumblocks, 2 );
     }// i
 
 }
@@ -213,16 +236,22 @@ void PixelMatcher::MatchPic(const PicArray& pic_data , const PicArray& ref_data
     // Initialisation //
     ////////////////////
 
+    m_big_xr = std::min( m_tdiff[ref_id-1], 3 )*m_encparams.XRangeME();
+    m_big_yr = std::min( m_tdiff[ref_id-1], 3 )*m_encparams.YRangeME();
+
     // Set the search ranges according to the level
-    if ( m_level == m_depth )
+    if ( m_encparams.FullSearch() == false )
     {
-        m_xr = 5;
-        m_yr = 5;
+        m_cost_mean = 0.0;
+        m_cost_mean_sq = 0.0;
+        
+        m_xr = std::min( m_level+1, 5);
+        m_yr = std::min( m_level+1, 5);
     }
     else
     {
-        m_xr = 1;
-        m_yr = 1;
+        m_xr = m_big_xr;
+        m_yr = m_big_yr;
     }
 
     // Provide aliases for the appropriate motion vector data components
@@ -230,14 +259,23 @@ void PixelMatcher::MatchPic(const PicArray& pic_data , const PicArray& ref_data
     MvArray& mv_array = me_data.Vectors( ref_id );
     const MvArray& guide_array = guide_data.Vectors( ref_id );
     TwoDArray<MvCostData>& pred_costs = me_data.PredCosts( ref_id );
+    
+    // Initialise the arrays
+    for (int y=0; y<mv_array.LengthY(); ++y)
+    {
+        for (int x=0; x<mv_array.LengthX(); ++x)
+        {
+    	    mv_array[y][x].x = 0;
+    	    mv_array[y][x].y = 0;
+    	    pred_costs[y][x].total = 10000000.0f;
+        }// x 
+    }// y 
 
     // Provide a block matching object to do the work
     BlockMatcher my_bmatch( pic_data , ref_data , 
                             m_encparams.LumaBParams(2) , m_encparams.MVPrecision() ,
                             mv_array , pred_costs );
 
-    float loc_lambda( 0.0 );
-
     // Do the work - loop over all the blocks, finding the best match //
     ////////////////////////////////////////////////////////////////////
 
@@ -253,10 +291,7 @@ void PixelMatcher::MatchPic(const PicArray& pic_data , const PicArray& ref_data
     m_cand_list.clear();
     MVector zero_mv( 0 , 0 );    
 
-    if (m_level<m_depth)
-        AddNewVlist( m_cand_list , zero_mv , m_xr , m_yr);
-    else
-        AddNewVlistD( m_cand_list , zero_mv , m_xr , m_yr);
+    AddNewVlist( m_cand_list , zero_mv , m_xr , m_yr);
 
     // Now loop over the blocks and find the best matches. 
     // The loop is unrolled because predictions are different at picture edges.
@@ -268,17 +303,13 @@ void PixelMatcher::MatchPic(const PicArray& pic_data , const PicArray& ref_data
     // Set the prediction as the zero vector
     m_mv_prediction = zero_mv;
 
-    // m_lambda is the Lagrangian smoothing parameter set to zero to get us started
-    m_lambda = 0.0;
-    DoBlock(0, 0 , pred_costs , mv_array, guide_array , my_bmatch);
+    DoBlock(0, 0 , guide_array , my_bmatch);
 
     // The rest of the first row
-    // ( use reduced lambda here )
-    m_lambda = loc_lambda / float( m_encparams.YNumBlocks() );
     for ( int xpos=1 ; xpos<mv_array.LengthX() ; ++xpos )
     {
         m_mv_prediction = mv_array[0][xpos-1];
-        DoBlock(xpos, 0 , pred_costs , mv_array, guide_array , my_bmatch);
+        DoBlock(xpos, 0 , guide_array , my_bmatch);
     }// xpos
 
     // All the remaining rows except the last 
@@ -287,53 +318,52 @@ void PixelMatcher::MatchPic(const PicArray& pic_data , const PicArray& ref_data
 
         // The first element of each row
         m_mv_prediction = mv_array[ypos-1][0];
-        m_lambda = loc_lambda/float(m_encparams.XNumBlocks());
-        DoBlock(0, ypos , pred_costs , mv_array, guide_array , my_bmatch );
+        DoBlock(0, ypos , guide_array , my_bmatch );
 
-         // The middle elementes of each row
-        m_lambda = loc_lambda;
+         // The middle elements of each row
         for ( int xpos=1 ; xpos<mv_array.LastX() ; ++xpos )
         {
             m_mv_prediction = MvMedian( mv_array[ypos][xpos-1],
                                         mv_array[ypos-1][xpos],
                                         mv_array[ypos-1][xpos+1]);
-            DoBlock(xpos, ypos , pred_costs , mv_array, guide_array , my_bmatch );
+            DoBlock(xpos, ypos , guide_array , my_bmatch );
 
         }// xpos
 
          // The last element in each row
-        m_lambda = loc_lambda/float( m_encparams.XNumBlocks() );
         m_mv_prediction = MvMean( mv_array[ypos-1][ mv_array.LastX() ],
                                   mv_array[ypos][ mv_array.LastX()-1 ]);
-        DoBlock(mv_array.LastX() , ypos , pred_costs , mv_array, guide_array , my_bmatch );
+        DoBlock(mv_array.LastX() , ypos , guide_array , my_bmatch );
     }//ypos
 
 }
 
 void PixelMatcher::DoBlock(const int xpos, const int ypos ,
-                           TwoDArray<MvCostData>& pred_costs,
-                           MvArray& mv_array,
                            const MvArray& guide_array,
                            BlockMatcher& block_match)
 {
-
     // Find the best match for each block ...
 
     // Use guide from lower down if one exists
     if ( m_level<m_depth )
-        AddNewVlist( m_cand_list , guide_array[ ypos>>1 ][ xpos>>1 ] * 2 , m_xr , m_yr );
+    {
+        int xdown = BChk(xpos>>1, guide_array.LengthX());
+        int ydown = BChk(ypos>>1, guide_array.LengthY());
+        AddNewVlist( m_cand_list , guide_array[ydown][xdown] * 2 , m_xr , m_yr );
+
+    }
 
     // use the spatial prediction, also, as a guide
-    if (m_level<m_depth)
-        AddNewVlist( m_cand_list , m_mv_prediction , m_xr , m_yr);
+    if (m_encparams.FullSearch()==false )
+        AddNewVlist( m_cand_list , m_mv_prediction , m_xr , m_yr );
     else
-        AddNewVlistD( m_cand_list , m_mv_prediction , m_xr , m_yr);
+        AddNewVlist( m_cand_list , m_mv_prediction , 1 , 1);
 
     // Find the best motion vector //
     /////////////////////////////////
 
-    block_match.FindBestMatchPel( xpos , ypos , m_cand_list, m_mv_prediction, m_lambda );
-
+    block_match.FindBestMatchPel( xpos , ypos , m_cand_list, m_mv_prediction, 0 );
+    
     // Reset the lists ready for the next block (don't erase the first sublist as
     // this is a neighbourhood of zero, which we always look at)
     m_cand_list.erase( m_cand_list.begin()+1 , m_cand_list.end() );
