@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: frames_test.cpp,v 1.1 2004/11/26 13:36:49 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: frames_test.cpp,v 1.6 2007/09/03 11:31:43 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -49,11 +49,8 @@ CPPUNIT_TEST_SUITE_NAMED_REGISTRATION (FramesTest, coreSuiteName());
 void FramesTest::setupFrame (Frame& frame, int start_val)
 {
     setupPicArray(frame.Ydata(), start_val);
-    if (frame.GetFparams().CFormat() != Yonly)
-    {
-        setupPicArray(frame.Udata(), start_val);
-        setupPicArray(frame.Vdata(), start_val);
-    }
+    setupPicArray(frame.Udata(), start_val);
+    setupPicArray(frame.Vdata(), start_val);
 }
 
 bool FramesTest::setupPicArray (PicArray &arr, int start_val)
@@ -85,11 +82,8 @@ bool FramesTest::setupPicArray (PicArray &arr, int start_val)
 void FramesTest::zeroFrame (Frame& frame)
 {
     zeroPicArray(frame.Ydata());
-    if (frame.GetFparams().CFormat() != Yonly)
-    {
-        zeroPicArray(frame.Udata());
-        zeroPicArray(frame.Vdata());
-    }
+    zeroPicArray(frame.Udata());
+    zeroPicArray(frame.Vdata());
 }
 
 bool FramesTest::zeroPicArray (PicArray &arr)
@@ -175,11 +169,10 @@ bool FramesTest::equalFrames (const Frame &lhs, const Frame &rhs)
 {
     CPPUNIT_ASSERT_EQUAL (lhs.GetFparams().CFormat(), rhs.GetFparams().CFormat() );
     CPPUNIT_ASSERT (equalPicArrays(lhs.Ydata(), rhs.Ydata()));
-    if (lhs.GetFparams().CFormat() != Yonly)
-    {
-        CPPUNIT_ASSERT (equalPicArrays(lhs.Udata(), rhs.Udata()));
-        CPPUNIT_ASSERT (equalPicArrays(lhs.Vdata(), rhs.Vdata()));
-    }
+    CPPUNIT_ASSERT (equalPicArrays(lhs.Udata(), rhs.Udata()));
+    CPPUNIT_ASSERT (equalPicArrays(lhs.Vdata(), rhs.Vdata()));
+    CPPUNIT_ASSERT_EQUAL (lhs.GetFparams().LumaDepth(), rhs.GetFparams().LumaDepth() );
+    CPPUNIT_ASSERT_EQUAL (lhs.GetFparams().ChromaDepth(), rhs.GetFparams().ChromaDepth() );
 
     return true;
 }
@@ -188,11 +181,8 @@ bool FramesTest::almostEqualFrames (const Frame &lhs, const Frame &rhs, int allo
 {
     CPPUNIT_ASSERT_EQUAL (lhs.GetFparams().CFormat(), rhs.GetFparams().CFormat() );
     CPPUNIT_ASSERT (almostEqualPicArrays(lhs.Ydata(), rhs.Ydata(), allowedError));
-    if (lhs.GetFparams().CFormat() != Yonly)
-    {
-        CPPUNIT_ASSERT (almostEqualPicArrays(lhs.Udata(), rhs.Udata(), allowedError));
-        CPPUNIT_ASSERT (almostEqualPicArrays(lhs.Vdata(), rhs.Vdata(), allowedError));
-    }
+    CPPUNIT_ASSERT (almostEqualPicArrays(lhs.Udata(), rhs.Udata(), allowedError));
+    CPPUNIT_ASSERT (almostEqualPicArrays(lhs.Vdata(), rhs.Vdata(), allowedError));
 
     return true;
 }
@@ -215,7 +205,7 @@ void FramesTest::tearDown()
 
 void FramesTest::testConstructor()
 {
-    FrameParams f_params(Yonly, 20, 30);
+    FrameParams f_params(format444, 20, 30, 20, 30, 20, 30, 8, 8);
     Frame frame(f_params);
 
     CPPUNIT_ASSERT_EQUAL (20, frame.Ydata().LengthX());
@@ -239,7 +229,7 @@ void FramesTest::testDefaultFParam()
 
 void FramesTest::testCopyConstructor()
 {
-    FrameParams f_params(Yonly, 20, 30);
+    FrameParams f_params(format444, 20, 30, 20, 30, 20, 30, 8, 8);
     Frame frame(f_params);
     setupFrame(frame, 0);
     
@@ -249,11 +239,11 @@ void FramesTest::testCopyConstructor()
 
 void FramesTest::testAssignment()
 {
-    FrameParams f_params(Yonly, 20, 30);
+    FrameParams f_params(format444, 20, 30, 20, 30, 20, 30, 8, 8);
     Frame frame(f_params);
     setupFrame(frame, 0);
 
-    FrameParams f_params_copy(Yonly,10,10);
+    FrameParams f_params_copy(format444, 10, 10, 10, 10, 10, 10, 8, 8);
     Frame frame_copy(f_params_copy);
 
     frame_copy = frame;
