@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: wavelet_utils_test.cpp,v 1.2 2005/05/05 14:10:08 tjdwave Exp $ $Name: Dirac_0_5_4 $
+* $Id: wavelet_utils_test.cpp,v 1.3 2007/07/26 12:53:59 tjdwave Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -92,13 +92,16 @@ void WaveletTransformTest::testTransformInvertibility()
 
     PicArray copy_data( pic_data );
 
+    // Array for storing the coefficients
+    CoeffArray coeff_data( pic_data.LengthY(), pic_data.LengthX() );
+
     for (int i=0 ; i< NUM_WLT_FILTERS; ++i)
     {
         WaveletTransform wtransform( depth , (WltFilter) i );
 
         // Go forward and back - we should be back where we started
-        wtransform.Transform( FORWARD , pic_data );
-        wtransform.Transform( BACKWARD , pic_data );
+        wtransform.Transform( FORWARD , pic_data, coeff_data );
+        wtransform.Transform( BACKWARD , pic_data, coeff_data );
 
         bool test_val = equalArrays<ValueType>( pic_data , copy_data );
 
