@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: instrmain.cpp,v 1.6 2005/05/20 12:43:53 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: instrmain.cpp,v 1.15 2008/01/15 04:36:24 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -59,7 +59,7 @@ static void DisplayHelp()
     cout << "\ninput                string  I  [ required ]  Input file name";
     cout << "\noutput               string  I  [ required ]  Output file name";
     cout << "\n";
-    cout << "\nmotion_colour        bool    I  true          Display motion vectors using colour wheel";    
+    cout << "\nmotion_colour        bool    I  true          Display motion vectors using colour wheel";
     cout << "\nmotion_arrows        bool    I  false         Display motion vectors as arrows";
     cout << "\nmotion_colour_arrows bool    I  false         Display motion vectors as arrows with colour size";
     cout << "\nsplit_mode           bool    I  false         Display macroblock splitting mode";
@@ -71,7 +71,7 @@ static void DisplayHelp()
     cout << "\nno_legend            bool    I  false         Do not display colour legend";
     cout << "\n";
     cout << "\nglobal               bool    I  false         Display global motion";
-    cout << "\nglobal_diff          bool    I  false         Display global motion error";    
+    cout << "\nglobal_diff          bool    I  false         Display global motion error";
     cout << "\nclip                 int     I  25 / 10000    Clip for max value motion vector / SAD overlays";
     cout << "\nref                  int     I  1             Reference frame";
     cout << "\nstart                int     I  0             Frame number at which process starts";
@@ -81,7 +81,7 @@ static void DisplayHelp()
     cout << "\nverbose              bool    I  false         Display information during process";
 }
 
-bool ReadSequenceParams (std::istream &in, SeqParams &seqparams)
+bool ReadInstrumentationHeader (std::istream &in, SourceParams& srcparams, bool &field_coding)
 {
     if (! in )
         return false;
@@ -90,23 +90,30 @@ bool ReadSequenceParams (std::istream &in, SeqParams &seqparams)
     bool temp_bool;
 
     in >> temp_int;
-    seqparams.SetCFormat( (ChromaFormat)temp_int );
+    srcparams.SetCFormat( (ChromaFormat)temp_int );
 
     in >> temp_int;
-    seqparams.SetXl( temp_int );
- 
+    srcparams.SetXl( temp_int );
+
     in >> temp_int;
-    seqparams.SetYl( temp_int );
+    srcparams.SetYl( temp_int );
+
+    in >> temp_int;
+    srcparams.SetSourceSampling( temp_int );
 
     in >> temp_bool;
-    seqparams.SetInterlace( temp_bool );
+    srcparams.SetTopFieldFirst( temp_bool );
 
-    in >> temp_bool;
-    seqparams.SetTopFieldFirst( temp_bool );
-
-    in >> temp_int;    
-    seqparams.SetFrameRate( temp_int );
+    int num, denom;
+    in >> num;
+    in >> denom;
+    srcparams.SetFrameRate( num, denom );
 
+    in >> num;
+    in >> denom;
+    srcparams.SetPixelAspectRatio( num, denom );
+    
+    in >> field_coding;
     return true;
 }
 
@@ -136,19 +143,19 @@ int main (int argc, char* argv[])
     bool_opts.insert("no_legend");
     bool_opts.insert("motion_colour");
     bool_opts.insert("motion_arrows");
-    bool_opts.insert("motion_colour_arrows");    
+    bool_opts.insert("motion_colour_arrows");
     bool_opts.insert("split_mode");
     bool_opts.insert("sad");
-    bool_opts.insert("pred_mode");    
+    bool_opts.insert("pred_mode");
     bool_opts.insert("global");
     bool_opts.insert("global_diff");
     bool_opts.insert("global_inliers");
-    
+
     // parse command line options
     CommandLine args(argc,argv,bool_opts);
 
     // need at least 3 arguments - the program name, an input and an output
-    if (argc < 3) 
+    if (argc < 3)
     {
         DisplayHelp();
         exit(1);
@@ -175,10 +182,10 @@ int main (int argc, char* argv[])
         {
             if (opt->m_name == "motion_arrows")
                 oparams.SetOption(motion_arrows);
-                
+
             else if (opt->m_name == "motion_colour_arrows")
                 oparams.SetOption(motion_colour_arrows);
-                
+
             else if (opt->m_name == "motion_colour")
                 oparams.SetOption(motion_colour);
 
@@ -207,29 +214,29 @@ int main (int argc, char* argv[])
             {
                 if (oparams.Option() == motion_arrows)
                     oparams.SetOption(gm_arrows);
-                    
+
                 if (oparams.Option() == motion_colour_arrows)
                     oparams.SetOption(gm_colour_arrows);
-                    
+
                 if (oparams.Option() == motion_colour)
                     oparams.SetOption(gm_colour);
             }
-            
+
             if (opt->m_name == "global_diff")
             {
                 if (oparams.Option() == motion_arrows
                     || oparams.Option() == gm_arrows)
                     oparams.SetOption(gm_diff_arrows);
-                    
+
                 if (oparams.Option() == motion_colour_arrows
                     || oparams.Option() == gm_colour_arrows)
                     oparams.SetOption(gm_diff_colour_arrows);
-                    
+
                 if (oparams.Option() == motion_colour
                     || oparams.Option() == gm_colour)
                     oparams.SetOption(gm_diff_colour);
             }
-            
+
         }
 
         // parameters
@@ -277,10 +284,10 @@ int main (int argc, char* argv[])
             if (opt->m_name == "buffer")
             {
                 buffer = strtoul(opt->m_value.c_str(),NULL,10);
-            } // m_name            
+            } // m_name
         } // opt
     } // args > 3
-    
+
     // read motion data from file
     if (verbose) cerr << endl << "Opening motion data file ";
     char mv_file[FILENAME_MAX];
@@ -288,31 +295,34 @@ int main (int argc, char* argv[])
     strcat(mv_file, ".imt");
     if (verbose) cerr << mv_file;
     ifstream in(mv_file, ios::in);
-    
+
     if (!in)
     {
         cerr << endl << "Failed to open sequence motion data file. Exiting." << endl;
         exit(EXIT_FAILURE);
     }
 
-    SeqParams seqparams;
-    ReadSequenceParams (in, seqparams);
-    // if the input sequence is Y only, make the ouput sequence 4:2:0
-    if (seqparams.CFormat() == Yonly)
-    {
-        seqparams.SetCFormat( format420 );
-    }
-
+    SourceParams srcparams;
+    bool field_coding; // true if material has been coded as fields and not frames
+    ReadInstrumentationHeader (in, srcparams, field_coding);
+    SourceParams out_srcparams(srcparams);
 
     // Create objects for input and output picture sequences
-    FileStreamInput inputpic(input.c_str(), seqparams);
+    char yuv_file[FILENAME_MAX];
+    strcpy(yuv_file, input.c_str());
+    strcat(yuv_file, ".localdec.yuv");
+    // hack hack - set interlace flag in source params to field_coding
+    // so that the Frame Parameters are set correctly.
+    srcparams.SetSourceSampling(field_coding);
+    FileStreamInput inputpic(yuv_file, srcparams, field_coding);
+
+    if (field_coding)
+        out_srcparams.SetYl(out_srcparams.Yl()>>1);
+    FileStreamOutput outputpic(output.c_str(), out_srcparams, false);
 
-    
-    FileStreamOutput outputpic(output.c_str(), seqparams);
-    
     if (verbose) cerr << " ... ok" << endl << "Processing sequence...";
     // *** process the sequence ***
-    ProcessSequence process(oparams, inputpic, outputpic, in, verbose, buffer, seqparams);
+    ProcessSequence process(oparams, inputpic, outputpic, in, verbose, buffer, srcparams);
     process.DoSequence(start, stop);
     if (verbose) cerr << endl << "Done sequence." << endl;
     return 0;
