@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: draw_overlay.cpp,v 1.4 2004/11/22 13:42:33 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: draw_overlay.cpp,v 1.8 2007/08/31 13:10:47 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -49,49 +49,49 @@ DrawOverlay::DrawOverlay(Frame & frame, DrawFrameMotionParams & draw_params)
 DrawOverlay::~DrawOverlay()
 {}
 
-// calculates U and V from a value ranging 0 ~ 1000 (normalising is carried out in calling function)
+// calculates U and V from a value ranging 0 ~ 250 (normalising is carried out in calling function)
 void DrawOverlay::GetPowerUV(int power, int & U, int & V)
 {
     // first convert power into RGB values
-    // ranging 0 ~ 500
+    // ranging 0 ~ 125
     float R = 0, G = 0, B = 0;
 
     // check which region value is located
-    if (power < 200)
+    if (power < 50)
     {
         B = 0;
-        G = 1000;
+        G = 250;
         R = 5 * power;
     }
-    else if (power >= 200 && power < 400)
+    else if (power >= 50 && power < 100)
     {
-        R = 1000;
+        R = 250;
         B = 0;
-        G = 1000 - (5 * (power - 200));
+        G = 250 - (5 * (power - 50));
     }
-    else if (power >= 400 && power < 600)
+    else if (power >= 100 && power < 150)
     {
-        R = 1000;
+        R = 250;
         G = 0;
-        B = 5 * (power - 400);
+        B = 5 * (power - 100);
     }
-    else if (power >= 600 && power < 800)
+    else if (power >= 150 && power < 200)
     {
-        B = 1000;
+        B = 250;
         G = 0;
-        R = 1000 - (5 * (power - 600));
+        R = 250 - (5 * (power - 150));
     }
-    else if (power >= 800 && power < 1000)
+    else if (power >= 200 && power < 250)
     {
-        B = 1000;
+        B = 250;
         R = 0;
-        G = 5 * (power - 800);
+        G = 5 * (power - 200);
     }
-    else if (power >= 1000)
+    else if (power >= 250)
     {
-        B = 1000;
+        B = 250;
         R = 0;
-        G = 1000;
+        G = 250;
     }
 
     R *= 0.25;
@@ -114,14 +114,14 @@ void DrawOverlay::DrawPowerBar(int min, int max)
         m_frame.Ydata()[ypx][5]=0;
 
         for (int xpx=0; xpx<5; ++xpx)
-            m_frame.Ydata()[ypx][xpx]=512; // grey background
+            m_frame.Ydata()[ypx][xpx]=0; // grey background
     }
 
     // draw colour on line by line basis
     for (int ypx=40/m_draw_params.ChromaFactorY(); ypx<(m_draw_params.PicY()/m_draw_params.ChromaFactorY()); ++ypx)
     {
         // find equivalent power value
-        double power = (1000 * (((m_draw_params.PicY()/m_draw_params.ChromaFactorY())) - (40/m_draw_params.ChromaFactorY()) - ypx) /
+        double power = (250 * (((m_draw_params.PicY()/m_draw_params.ChromaFactorY())) - (40/m_draw_params.ChromaFactorY()) - ypx) /
                         (m_draw_params.PicY()/m_draw_params.ChromaFactorY())-(40/m_draw_params.ChromaFactorY()));
 
         // get U V values for power
@@ -130,8 +130,8 @@ void DrawOverlay::DrawPowerBar(int min, int max)
 
         for (int xpx=0; xpx<=4/m_draw_params.ChromaFactorX(); ++xpx)
         {
-            m_frame.Udata()[ypx][xpx]=U+512;
-            m_frame.Vdata()[ypx][xpx]=V+512;
+            m_frame.Udata()[ypx][xpx]=U;
+            m_frame.Vdata()[ypx][xpx]=V;
         }
     }
 
@@ -149,7 +149,7 @@ void DrawOverlay::DrawCharacter(const PicArray & ch, int y_offset, int x_offset)
     {
         for (int x=x_offset, x_ch=0; x<x_offset+8; ++x, ++x_ch)
         {
-            m_frame.Ydata()[y][x]=ch[y_ch][x_ch]*1024;
+            m_frame.Ydata()[y][x]=ch[y_ch][x_ch]*255-128;
         }// x
     }// y
 
@@ -158,8 +158,8 @@ void DrawOverlay::DrawCharacter(const PicArray & ch, int y_offset, int x_offset)
     {
         for (int xpx=x_offset/m_draw_params.ChromaFactorX(); xpx<(x_offset+8)/m_draw_params.ChromaFactorX(); ++xpx)
         {
-            m_frame.Udata()[ypx][xpx]=512;
-            m_frame.Vdata()[ypx][xpx]=512;
+            m_frame.Udata()[ypx][xpx]=0;
+            m_frame.Vdata()[ypx][xpx]=0;
         }// xpx
     }// ypx
 }
@@ -167,7 +167,7 @@ void DrawOverlay::DrawCharacter(const PicArray & ch, int y_offset, int x_offset)
 // draws value in luma
 void DrawOverlay::DrawValue(int number, int y_offset, int x_offset)
 {
-    int digits;
+    int digits = 0;
     // number of digits in frame number
     if (number < 10)
         digits = 1;
@@ -183,7 +183,7 @@ void DrawOverlay::DrawValue(int number, int y_offset, int x_offset)
     // loop over digits
     for (int digit=digits; digit>0; --digit)
     {
-        int value;
+        int value = 0;
 
         // get digit, largest first
         if (digit == 5)
@@ -284,10 +284,21 @@ void DrawOverlay::DrawMvBlockUV(int ymv, int xmv, int U, int V)
     // loop over chroma samples in block
     for (int y=0; y<m_draw_params.MvUVBlockY(); ++y)
     {
+        int y_idx = (ymv*m_draw_params.MvUVBlockY())+y;
+        if (y_idx >= m_frame.Udata().LengthY() || 
+            y_idx >= m_frame.Vdata().LengthY())
+            break;
         for (int x=0; x<m_draw_params.MvUVBlockX(); ++x)
         {
-            m_frame.Udata()[(ymv*m_draw_params.MvUVBlockY())+y][(xmv*m_draw_params.MvUVBlockX())+x]=U;
-            m_frame.Vdata()[(ymv*m_draw_params.MvUVBlockY())+y][(xmv*m_draw_params.MvUVBlockX())+x]=V;
+            int x_idx = (xmv*m_draw_params.MvUVBlockX())+x;
+            if (x_idx >= m_frame.Udata().LengthX() || 
+                x_idx >= m_frame.Vdata().LengthX())
+                break;
+              
+            //m_frame.Udata()[(ymv*m_draw_params.MvUVBlockY())+y][(xmv*m_draw_params.MvUVBlockX())+x]=U;
+            //m_frame.Vdata()[(ymv*m_draw_params.MvUVBlockY())+y][(xmv*m_draw_params.MvUVBlockX())+x]=V;
+            m_frame.Udata()[y_idx][x_idx]=U;
+            m_frame.Vdata()[y_idx][x_idx]=V;
         }// xpx
     }// ypx
 }
