@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: motion_arrows.cpp,v 1.3 2004/11/22 13:42:33 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: motion_arrows.cpp,v 1.6 2007/08/31 13:10:47 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -59,10 +59,14 @@ void DrawMotionArrows::DrawBlock(int j, int i)
     // no chroma in frame
     for (int y=j*m_draw_params.MvUVBlockY(); y<(j+1)*m_draw_params.MvUVBlockY(); ++y)
     {
+        if (y >= m_frame.Udata().LengthY() || y >= m_frame.Vdata().LengthY())
+            break;
         for (int x=i*m_draw_params.MvUVBlockX(); x<(i+1)*m_draw_params.MvUVBlockX(); ++x)
         {
-            m_frame.Udata()[y][x] = 512;
-            m_frame.Vdata()[y][x] = 512;
+            if (x >= m_frame.Udata().LengthX() || x >= m_frame.Vdata().LengthX())
+                break;
+            m_frame.Udata()[y][x] = 0;
+            m_frame.Vdata()[y][x] = 0;
         }
     }
 
@@ -192,7 +196,7 @@ void DrawMotionArrows::DrawArrow(int j, int i, int y_pos, int x_pos)
         {
             for (int xpx=0; xpx<16; ++xpx)
             {
-                m_frame.Ydata()[(y_pos)+ypx][(x_pos) + xpx] += m_symbols.Arrow()[ypx][15-xpx] * 1024;
+                m_frame.Ydata()[(y_pos)+ypx][(x_pos) + xpx] += m_symbols.Arrow()[ypx][15-xpx] * 256;
             }// xpx
         }// ypx
     }
@@ -202,7 +206,7 @@ void DrawMotionArrows::DrawArrow(int j, int i, int y_pos, int x_pos)
         {
             for (int xpx=0; xpx<16; ++xpx)
             {
-                m_frame.Ydata()[(y_pos) + ypx][(x_pos) + xpx] += m_symbols.Arrow()[15-ypx][xpx] * 1024;
+                m_frame.Ydata()[(y_pos) + ypx][(x_pos) + xpx] += m_symbols.Arrow()[15-ypx][xpx] * 256;
             }// xpx
         }// ypx
     }
@@ -212,7 +216,7 @@ void DrawMotionArrows::DrawArrow(int j, int i, int y_pos, int x_pos)
         {
             for (int xpx=0; xpx<16; ++xpx)
             {
-                m_frame.Ydata()[(y_pos) + ypx][(x_pos) + xpx] += m_symbols.Arrow()[15-ypx][15-xpx] * 1024;
+                m_frame.Ydata()[(y_pos) + ypx][(x_pos) + xpx] += m_symbols.Arrow()[15-ypx][15-xpx] * 256;
             }// xpx
         }// ypx
     }
@@ -222,7 +226,7 @@ void DrawMotionArrows::DrawArrow(int j, int i, int y_pos, int x_pos)
         {
             for (int xpx=0; xpx<16; ++xpx)
             {
-                m_frame.Ydata()[(y_pos) + ypx][(x_pos) + xpx] += m_symbols.Arrow()[ypx][xpx] * 1024;
+                m_frame.Ydata()[(y_pos) + ypx][(x_pos) + xpx] += m_symbols.Arrow()[ypx][xpx] * 256;
             }// xpx
         }// ypx
     }
