@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: motion_colour.cpp,v 1.3 2004/11/22 13:42:33 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: motion_colour.cpp,v 1.5 2007/08/31 13:10:47 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -55,8 +55,8 @@ DrawMotionColour::~DrawMotionColour()
 // colours motion vector block appropriately
 void DrawMotionColour::DrawBlock(int j, int i)
 {
-    DrawMvBlockUV(j, i, (((m_mv[j][i].x) * (500/m_mv_clip)) / m_mv_scale) + 500,
-                        (((m_mv[j][i].y) * (500/m_mv_clip)) / m_mv_scale) + 500);
+    DrawMvBlockUV(j, i, (((m_mv[j][i].x) * (125/m_mv_clip)) / m_mv_scale),
+                        (((m_mv[j][i].y) * (125/m_mv_clip)) / m_mv_scale));
 }
 
 // draws colour wheel legend
@@ -69,11 +69,11 @@ void DrawMotionColour::DrawLegend()
         // white background
         for (int xpx=1; xpx<40; ++xpx)
         {
-            m_frame.Ydata()[ypx][xpx]=500;
+            m_frame.Ydata()[ypx][xpx]=0;
         }
 
         // crosshair vertical line
-        m_frame.Ydata()[ypx][21]=350;
+        m_frame.Ydata()[ypx][21]=88-128;
     }
 
     // colour in the rectangle
@@ -83,22 +83,22 @@ void DrawMotionColour::DrawLegend()
     {
         for (int xpx=40/m_draw_params.ChromaFactorX(), x=20; xpx>=0; --xpx, x-=m_draw_params.ChromaFactorX())
         {
-            m_frame.Udata()[ypx][xpx]=(x*25)+500;
-            m_frame.Vdata()[ypx][xpx]=(y*25)+500;
+            m_frame.Udata()[ypx][xpx]=(x*25)-128;
+            m_frame.Vdata()[ypx][xpx]=(y*25)-128;
         }
     }
 
     // crosshair horizontal linem_
     for (int xpx=0; xpx<40; ++xpx)
     {
-        m_frame.Ydata()[y_start+16][xpx]=350;
+        m_frame.Ydata()[y_start+16][xpx]=88-128;
     }
 
     // vertical black line
     for (int ypx=y_start+1; ypx<=y_start+30; ++ypx)
     {
-        m_frame.Ydata()[ypx][41]=0;
-        m_frame.Ydata()[ypx][0]=0;
+        m_frame.Ydata()[ypx][41]=-128;
+        m_frame.Ydata()[ypx][0]=-128;
     }
 
     // horizontal black line
