@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: overlay.cpp,v 1.6 2004/11/22 13:42:33 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: overlay.cpp,v 1.9 2007/08/31 13:10:47 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -62,7 +62,7 @@ void Overlay::ProcessFrame()
         for (int j=0; j<=m_frame.Ydata().LastY(); ++j)
         {
             for (int i=0; i<=m_frame.Ydata().LastX(); ++i)
-                m_frame.Ydata()[j][i]=512;
+                m_frame.Ydata()[j][i]=0;
         }
     }
 
@@ -71,8 +71,8 @@ void Overlay::ProcessFrame()
     {
         for (int i=0; i<m_frame.Udata().LengthX(); ++i)
         {
-            m_frame.Udata()[j][i]=512;
-            m_frame.Vdata()[j][i]=512;
+            m_frame.Udata()[j][i]=0;
+            m_frame.Vdata()[j][i]=0;
         }
     }
 
@@ -104,7 +104,7 @@ void Overlay::ProcessFrame(const MEData & me_data, const OLBParams & block_param
         for (int j=0; j<=m_frame.Ydata().LastY(); ++j)
         {
             for (int i=0; i<=m_frame.Ydata().LastX(); ++i)
-                m_frame.Ydata()[j][i]=512;
+                m_frame.Ydata()[j][i]=0;
         }
     }
 
@@ -258,8 +258,8 @@ void Overlay::DoOverlay(const MEData & me_data)
         {
             for (int x=0; x<m_frame.Udata().LengthX(); ++x)
             {
-                m_frame.Udata()[y][x] = 512;
-                m_frame.Vdata()[y][x] = 512;
+                m_frame.Udata()[y][x] = 0;
+                m_frame.Vdata()[y][x] = 0;
             }
         }
         
@@ -299,12 +299,7 @@ void Overlay::DoOverlay(const MEData & me_data)
 // calculates the resolution factor between chroma and luma samples
 void Overlay::CalculateFactors(const ChromaFormat & cformat)
 {
-    if (cformat == Yonly)
-    {
-        m_draw_params.SetChromaFactorY(1);
-        m_draw_params.SetChromaFactorX(1);
-    }
-    else if (cformat == format422)
+    if (cformat == format422)
     {
         m_draw_params.SetChromaFactorY(1);
         m_draw_params.SetChromaFactorX(2);
@@ -319,11 +314,6 @@ void Overlay::CalculateFactors(const ChromaFormat & cformat)
         m_draw_params.SetChromaFactorY(1);
         m_draw_params.SetChromaFactorX(1);
     }
-    else if (cformat == format411)
-    {
-        m_draw_params.SetChromaFactorY(1);
-        m_draw_params.SetChromaFactorX(4);
-    }
     else
     {
         m_draw_params.SetChromaFactorY(1);
