@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: sad.cpp,v 1.3 2004/11/22 13:42:33 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: sad.cpp,v 1.6 2007/08/31 13:10:47 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -57,23 +57,31 @@ void DrawSad::DrawBlock(int j, int i)
 {
     // get U and V for motion vector block
     int U = 0, V = 0;
-    int value = int(m_cost[j][i].SAD / (double(m_scale) / 1000));
+    int value = int(m_cost[j][i].SAD / (double(m_scale) / 250));
     GetPowerUV(value, U, V);
-    DrawMvBlockUV(j, i, U+500, V+500);
+    DrawMvBlockUV(j, i, U, V);
 
     // if intra, draw white box round block
     if (m_mode[j][i]==dirac::INTRA)
     {
+        int yidx = (j*m_draw_params.MvYBlockY());
+        int xidx = (i*m_draw_params.MvYBlockX());
         for (int ypx=0; ypx<m_draw_params.MvYBlockY(); ++ypx)
         {
-            m_frame.Ydata()[(j*m_draw_params.MvYBlockY())+ypx][(i*m_draw_params.MvYBlockX())] = 1000;
-            m_frame.Ydata()[(j*m_draw_params.MvYBlockY())+ypx][(i*m_draw_params.MvYBlockX())+m_draw_params.MvYBlockX()-1] = 1000;
+            if ((yidx+ypx)>=m_frame.Ydata().LengthY() ||
+                (xidx+m_draw_params.MvYBlockX()-1)>= m_frame.Ydata().LengthX())
+                break;
+            m_frame.Ydata()[(j*m_draw_params.MvYBlockY())+ypx][(i*m_draw_params.MvYBlockX())] = 250;
+            m_frame.Ydata()[(j*m_draw_params.MvYBlockY())+ypx][(i*m_draw_params.MvYBlockX())+m_draw_params.MvYBlockX()-1] = 250;
         }// ypx
 
         for (int xpx=0; xpx<m_draw_params.MvYBlockX(); ++xpx)
         {
-            m_frame.Ydata()[(j*m_draw_params.MvYBlockY())][(i*m_draw_params.MvYBlockX())+xpx] = 1000;
-            m_frame.Ydata()[(j*m_draw_params.MvYBlockY())+m_draw_params.MvYBlockY()-1][(i*m_draw_params.MvYBlockX())+xpx] = 1000;
+            if ((yidx+m_draw_params.MvYBlockY()-1)>=m_frame.Ydata().LengthY() ||
+                (xidx+xpx)>= m_frame.Ydata().LengthX())
+                break;
+            m_frame.Ydata()[(j*m_draw_params.MvYBlockY())][(i*m_draw_params.MvYBlockX())+xpx] = 250;
+            m_frame.Ydata()[(j*m_draw_params.MvYBlockY())+m_draw_params.MvYBlockY()-1][(i*m_draw_params.MvYBlockX())+xpx] = 250;
         }// xpx
     }
 }
