@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: split_mode.cpp,v 1.4 2004/11/22 13:42:33 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: split_mode.cpp,v 1.6 2007/08/31 13:10:47 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -57,12 +57,12 @@ void DrawSplitMode::DrawBlock(int j, int i)
     if (m_mode[int(j/4)][int(i/4)] == 0)
         power=0;
     else if (m_mode[int(j/4)][int(i/4)] == 1)
-        power=400;
+        power=100;
     else if (m_mode[int(j/4)][int(i/4)] == 2)
-        power=800;
+        power=200;
 
     GetPowerUV(power, U, V);
-    DrawMvBlockUV(j, i, U+500, V+500);
+    DrawMvBlockUV(j, i, U, V);
 
 }
 
@@ -73,29 +73,29 @@ void DrawSplitMode::DrawLegend()
     for (int ypx=m_draw_params.PicY()-49; ypx<m_draw_params.PicY(); ++ypx)
     {
         for (int xpx=7; xpx>=0; --xpx)
-            m_frame.Ydata()[ypx][xpx]=500; // grey
+            m_frame.Ydata()[ypx][xpx]=0; // grey
     }
 
     int U=0, V=0;
 
-    GetPowerUV(800, U, V); // mode 2
-    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(48/m_draw_params.ChromaFactorY())+1, 0, U+500, V+500);
-    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(40/m_draw_params.ChromaFactorY())+1, 0, U+500, V+500);
+    GetPowerUV(200, U, V); // mode 2
+    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(48/m_draw_params.ChromaFactorY())+1, 0, U, V);
+    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(40/m_draw_params.ChromaFactorY())+1, 0, U, V);
     
-    GetPowerUV(400, U, V); // mode 1
-    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(32/m_draw_params.ChromaFactorY())+1, 0, U+500, V+500);
-    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(24/m_draw_params.ChromaFactorY())+1, 0, U+500, V+500);
+    GetPowerUV(100, U, V); // mode 1
+    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(32/m_draw_params.ChromaFactorY())+1, 0, U, V);
+    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(24/m_draw_params.ChromaFactorY())+1, 0, U, V);
     
     GetPowerUV(0, U, V); // mode 0
-    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(16/m_draw_params.ChromaFactorY())+1, 0, U+500, V+500);
-    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(8/m_draw_params.ChromaFactorY())+1, 0, U+500, V+500);
+    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(16/m_draw_params.ChromaFactorY())+1, 0, U, V);
+    DrawBlockUV((m_draw_params.PicY()/m_draw_params.ChromaFactorY())-1-(8/m_draw_params.ChromaFactorY())+1, 0, U, V);
 
     // black horizontal lines
     for (int xpx=15; xpx>=0; --xpx)
     {
-        m_frame.Ydata()[m_draw_params.PicY()-49][xpx]=0;
-        m_frame.Ydata()[m_draw_params.PicY()-33][xpx]=0;
-        m_frame.Ydata()[m_draw_params.PicY()-17][xpx]=0;
+        m_frame.Ydata()[m_draw_params.PicY()-49][xpx]=-128;
+        m_frame.Ydata()[m_draw_params.PicY()-33][xpx]=-128;
+        m_frame.Ydata()[m_draw_params.PicY()-17][xpx]=-128;
     }
 
     // draw '2 1 0' label
