@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: process_sequence.cpp,v 1.5 2005/08/01 16:11:48 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: process_sequence.cpp,v 1.10 2007/09/26 12:18:43 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -20,7 +20,8 @@
 * Portions created by the Initial Developer are Copyright (C) 2004.
 * All Rights Reserved.
 *
-* Contributor(s): Chris Bowley (Original Author)
+* Contributor(s): Chris Bowley (Original Author),
+*                 Tim Borer
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 (the "GPL"), or the GNU Lesser
@@ -40,18 +41,18 @@
 #include <libdirac_common/frame.h>
 using namespace dirac;
 
-ProcessSequence::ProcessSequence(OverlayParams & oparams, 
+ProcessSequence::ProcessSequence(OverlayParams & oparams,
                                  FileStreamInput & inputpic,
                                  FileStreamOutput & outputpic,
-                                 std::ifstream & in, bool verbose, 
-                                 int buffer, SeqParams & seqparams) :
+                                 std::ifstream & in, bool verbose,
+                                 int buffer, SourceParams & srcparams) :
     m_oparams(oparams),
     m_inputpic(inputpic),
     m_outputpic(outputpic),
     m_verbose(verbose),
     m_data_in(in),
     m_data_array(buffer),
-    m_seqparams(seqparams)
+    m_srcparams(srcparams)
 {
 
 }
@@ -67,11 +68,11 @@ bool ProcessSequence::DoFrame()
     {
         // read next frame from input sequence
         Frame * frame = new Frame(m_data_array[index].frame_params);
-        m_inputpic.ReadNextFrame(*frame);
+        m_inputpic.GetStream()->ReadNextPicture(*frame);
 
         Overlay overlay(m_oparams, *frame);
 
-        if (m_data_array[index].frame_params.FSort() == I_frame)
+        if (m_data_array[index].frame_params.FSort().IsIntra())
             overlay.ProcessFrame();
 
         else
@@ -91,7 +92,7 @@ bool ProcessSequence::DoFrame()
         frame->Clip();
 
         // write the frame to the output file
-        m_outputpic.WriteNextFrame(*frame);
+        m_outputpic.GetStream()->WriteNextFrame(*frame);
 
         // de-allocate memory for frame
         delete frame;
@@ -117,17 +118,17 @@ void ProcessSequence::AddFrameEntry()
     // reading information for an intra frame
     if (strcmp(mv_frame_sort, "intra") == 0)
     {
-        if (m_verbose) std::cerr << std::endl << "Reading intra frame " << m_data_fnum << " data";
+        if (m_verbose) std::cout << std::endl << "Reading intra frame " << m_data_fnum << " data";
 
         m_data_array[new_index].me_data = 0;
-        m_data_array[new_index].frame_params = m_seqparams;
+        m_data_array[new_index].frame_params = m_srcparams;
         m_data_array[new_index].frame_params.SetFrameNum(m_data_fnum);
-        m_data_array[new_index].frame_params.SetFSort(I_frame);
+        m_data_array[new_index].frame_params.SetFSort(FrameSort::IntraRefFrameSort());
 
         if (m_verbose)
         {
-            std::cerr << std::endl << "Writing to array position ";
-            std::cerr << m_data_fnum % m_data_array.Length();
+            std::cout << std::endl << "Writing to array position ";
+            std::cout << m_data_fnum % m_data_array.Length();
         }
     }
 
@@ -136,23 +137,23 @@ void ProcessSequence::AddFrameEntry()
     {
         if (m_verbose)
         {
-            std::cerr << std::endl << "Reading motion-compensated frame ";
-            std::cerr << m_data_fnum << " data";
+            std::cout << std::endl << "Reading motion-compensated frame ";
+            std::cout << m_data_fnum << " data";
         }
-        
+
         int mb_xnum = 0, mb_ynum = 0, mv_xnum = 0, mv_ynum = 0;
         int total_refs = 0;
         int ref = -1;
 
         // create frame motion data array entry
-        m_data_array[new_index].frame_params = m_seqparams;
+        m_data_array[new_index].frame_params = m_srcparams;
 
         // read reference frame information from top of file
         m_data_in >> total_refs;
 
         // clear reference vector
         m_data_array[new_index].frame_params.Refs().clear();
-        
+
         for (int i=0; i<total_refs; ++i)
         {
             m_data_in >> ref;
@@ -173,23 +174,22 @@ void ProcessSequence::AddFrameEntry()
         m_data_in >> mv_xnum;
 
         // create motion data object
-        // *** remove comments around last argument when encoder uses variable number of references ***
-        m_data_array[new_index].me_data = new MEData(mb_xnum, mb_ynum, mv_xnum, mv_ynum /*, total_refs */);
+        m_data_array[new_index].me_data = new MEData(mb_xnum, mb_ynum, mv_xnum, mv_ynum , total_refs );
 
         m_data_array[new_index].frame_params.SetFrameNum(m_data_fnum);
 
         if (m_data_array[new_index].frame_params.Refs().size() > 1)
-            m_data_array[new_index].frame_params.SetFSort(L2_frame);
+            m_data_array[new_index].frame_params.SetFSort(FrameSort::InterNonRefFrameSort());
         else
-            m_data_array[new_index].frame_params.SetFSort(L1_frame);
+            m_data_array[new_index].frame_params.SetFSort(FrameSort::InterRefFrameSort());
 
         // read motion vector data
-        m_data_in >> *m_data_array[new_index].me_data; // overloaded operator>> defined in libdirac_common/motion.cpp      
-        
+        m_data_in >> *m_data_array[new_index].me_data; // overloaded operator>> defined in libdirac_common/motion.cpp
+
         if (m_verbose)
         {
-            std::cerr << std::endl << "Writing to array position ";
-            std::cerr << m_data_fnum % m_data_array.Length();
+            std::cout << std::endl << "Writing to array position ";
+            std::cout << m_data_fnum % m_data_array.Length();
         }
     }
 }
@@ -212,9 +212,9 @@ void ProcessSequence::DoSequence(int start, int stop)
     {
         for (int fnum=0; fnum<start; ++fnum)
         {
-            FrameParams fparams(m_inputpic.GetSeqParams());
+            FrameParams fparams(m_inputpic.GetSourceParams());
             Frame * frame = new Frame(fparams);
-            m_inputpic.ReadNextFrame(*frame);
+            m_inputpic.GetStream()->ReadNextPicture(*frame);
             delete frame;
         }
     }
@@ -233,18 +233,18 @@ void ProcessSequence::DoSequence(int start, int stop)
     // frame by frame processing
     for (m_process_fnum = start; m_process_fnum <= stop; ++m_process_fnum)
     {
-        if (m_verbose) std::cerr << std::endl << std::endl << "Frame " << m_process_fnum;
+        if (m_verbose) std::cout << std::endl << std::endl << "Frame " << m_process_fnum;
 
         // location of frame data in array
         int index = int(m_process_fnum % m_data_array.Length());
 
         if (m_verbose)
         {
-            std::cerr << "\nArray entry " << index << " is ";
+            std::cout << "\nArray entry " << index << " is ";
             if (m_data_array[index].frame_params.FrameNum() != -1)
-                std::cerr << "frame number " << m_data_array[index].frame_params.FrameNum();
+                std::cout << "frame number " << m_data_array[index].frame_params.FrameNum();
             else
-                std::cerr << "not allocated";
+                std::cout << "not allocated";
         }
 
         // if the frame motion data has not already been read, add the motion data to the vector
@@ -275,10 +275,10 @@ void ProcessSequence::DoSequence(int start, int stop)
                 if (m_data_fnum == data_next_fnum && !m_data_in.eof())
                 {
                     m_data_fnum = data_next_fnum;
-                    if (m_verbose) std::cerr << std::endl << "Updating frame data";
+                    if (m_verbose) std::cout << std::endl << "Updating frame data";
                     AddFrameEntry();
                 }
-                               
+
             } while (m_data_fnum == data_next_fnum && !m_data_in.eof());
 
 
@@ -305,5 +305,5 @@ void ProcessSequence::DoSequence(int start, int stop)
 
     // close motion data file
     m_data_in.close();
-} 
+}
 
