@@ -1,6 +1,6 @@
 /* ***** BEGIN LICENSE BLOCK *****
 *
-* $Id: process_sequence.h,v 1.3 2004/11/22 13:39:48 asuraparaju Exp $ $Name: Dirac_0_5_4 $
+* $Id: process_sequence.h,v 1.4 2007/09/03 11:31:43 asuraparaju Exp $ $Name: Dirac_0_9_1 $
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
@@ -43,7 +43,7 @@
 #include <libdirac_common/common.h>
 using dirac::FrameParams;
 using dirac::OLBParams;
-using dirac::SeqParams;
+using dirac::SourceParams;
 using dirac::OneDArray;
 
 #include <libdirac_common/motion.h>
@@ -71,7 +71,7 @@ class ProcessSequence
 public :
 
     //! Constructor
-    ProcessSequence(OverlayParams &, FileStreamInput &, FileStreamOutput &, std::ifstream &, bool, int, SeqParams &);
+    ProcessSequence(OverlayParams &, FileStreamInput &, FileStreamOutput &, std::ifstream &, bool, int, SourceParams &);
 
     //! Destructor
     ~ProcessSequence() {}
@@ -121,8 +121,8 @@ private :
     //! Read input frame data signal
     bool used_frame_data;
 
-    //! Output sequence parameters
-    SeqParams & m_seqparams;
+    //! Output source parameters
+    SourceParams & m_srcparams;
 
 };
 
